# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer


class CognitiveServicesNetworkRulesTests(ScenarioTest):
    @ResourceGroupPreparer()
    def test_cognitiveservices_network_rules(self, resource_group):
        sname = self.create_random_name(prefix='cs_cli_test_', length=16)
        customdomain = self.create_random_name(prefix='csclitest', length=16)

        self.kwargs.update({
            'sname': sname,
            'vnetname': sname,
            'kind': 'Face',
            'sku': 'S0',
            'location': 'westus',
            'customdomain': customdomain,
        })

        self.cmd('network vnet create --resource-group {rg} --name {vnetname}')

        subnet1 = self.cmd('network vnet subnet create --resource-group {rg} --name default'
                           ' --vnet-name {vnetname} --address-prefixes 10.0.0.0/24').get_output_in_json()
        subnet2 = self.cmd('network vnet subnet create --resource-group {rg} --name subnet'
                           ' --vnet-name {vnetname} --address-prefixes 10.0.1.0/24').get_output_in_json()

        self.cmd('az cognitiveservices account create -n {sname} -g {rg} --kind {kind} --sku {sku} -l {location}'
                 ' --custom-domain {customdomain} --yes',
                 checks=[self.check('name', '{sname}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}'),
                         self.check('properties.provisioningState', 'Succeeded')])

        rules = self.cmd('az cognitiveservices account network-rule list -n {sname} -g {rg}').get_output_in_json()
        self.assertEqual(len(rules['ipRules']), 0)
        self.assertEqual(len(rules['virtualNetworkRules']), 0)

        self.cmd('az cognitiveservices account network-rule add -n {sname} -g {rg} --ip-address "200.0.0.1"')
        rules = self.cmd('az cognitiveservices account network-rule list -n {sname} -g {rg}').get_output_in_json()
        self.assertEqual(len(rules['ipRules']), 1)
        self.assertEqual(len(rules['virtualNetworkRules']), 0)
        self.assertEqual(rules['ipRules'][0]['value'], "200.0.0.1")

        self.cmd('az cognitiveservices account network-rule add -n {sname} -g {rg} --ip-address "100.0.0.0/24"')
        rules = self.cmd('az cognitiveservices account network-rule list -n {sname} -g {rg}').get_output_in_json()
        self.assertEqual(len(rules['ipRules']), 2)
        self.assertEqual(len(rules['virtualNetworkRules']), 0)
        self.assertEqual(rules['ipRules'][0]['value'], "200.0.0.1")
        self.assertEqual(rules['ipRules'][1]['value'], "100.0.0.0/24")

        self.cmd('az cognitiveservices account network-rule add -n {sname} -g {rg} --subnet ' + subnet1['id'])
        rules = self.cmd('az cognitiveservices account network-rule list -n {sname} -g {rg}').get_output_in_json()
        self.assertEqual(len(rules['ipRules']), 2)
        self.assertEqual(len(rules['virtualNetworkRules']), 1)
        self.assertEqual(rules['ipRules'][0]['value'], "200.0.0.1")
        self.assertEqual(rules['ipRules'][1]['value'], "100.0.0.0/24")
        self.assertEqual(rules['virtualNetworkRules'][0]['id'], subnet1['id'])

        self.cmd('az cognitiveservices account network-rule add -n {sname} -g {rg} --subnet ' + subnet2['name'] +
                 ' --vnet-name {vnetname}')
        rules = self.cmd('az cognitiveservices account network-rule list -n {sname} -g {rg}').get_output_in_json()
        self.assertEqual(len(rules['ipRules']), 2)
        self.assertEqual(len(rules['virtualNetworkRules']), 2)
        self.assertEqual(rules['ipRules'][0]['value'], "200.0.0.1")
        self.assertEqual(rules['ipRules'][1]['value'], "100.0.0.0/24")
        self.assertEqual(rules['virtualNetworkRules'][0]['id'], subnet1['id'])
        self.assertEqual(rules['virtualNetworkRules'][1]['id'], subnet2['id'])

        self.cmd('az cognitiveservices account network-rule remove -n {sname} -g {rg} --ip-address "200.0.0.1"')
        rules = self.cmd('az cognitiveservices account network-rule list -n {sname} -g {rg}').get_output_in_json()
        self.assertEqual(len(rules['ipRules']), 1)
        self.assertEqual(len(rules['virtualNetworkRules']), 2)
        self.assertEqual(rules['ipRules'][0]['value'], "100.0.0.0/24")
        self.assertEqual(rules['virtualNetworkRules'][0]['id'], subnet1['id'])
        self.assertEqual(rules['virtualNetworkRules'][1]['id'], subnet2['id'])

        self.cmd('az cognitiveservices account network-rule remove -n {sname} -g {rg} --ip-address "100.0.0.0/24"')
        rules = self.cmd('az cognitiveservices account network-rule list -n {sname} -g {rg}').get_output_in_json()
        self.assertEqual(len(rules['ipRules']), 0)
        self.assertEqual(len(rules['virtualNetworkRules']), 2)
        self.assertEqual(rules['virtualNetworkRules'][0]['id'], subnet1['id'])
        self.assertEqual(rules['virtualNetworkRules'][1]['id'], subnet2['id'])

        self.cmd('az cognitiveservices account network-rule remove -n {sname} -g {rg} --subnet ' + subnet1['id'])
        rules = self.cmd('az cognitiveservices account network-rule list -n {sname} -g {rg}').get_output_in_json()
        self.assertEqual(len(rules['ipRules']), 0)
        self.assertEqual(len(rules['virtualNetworkRules']), 1)
        self.assertEqual(rules['virtualNetworkRules'][0]['id'], subnet2['id'])

        self.cmd('az cognitiveservices account network-rule remove -n {sname} -g {rg} --subnet ' + subnet2['name'] +
                 ' --vnet-name {vnetname}')
        rules = self.cmd('az cognitiveservices account network-rule list -n {sname} -g {rg}').get_output_in_json()
        self.assertEqual(len(rules['ipRules']), 0)
        self.assertEqual(len(rules['virtualNetworkRules']), 0)

        # Remove something doesn't exists in rules
        self.cmd('az cognitiveservices account network-rule remove -n {sname} -g {rg} --subnet ' + subnet2['name'] +
                 ' --vnet-name {vnetname}')
        rules = self.cmd('az cognitiveservices account network-rule list -n {sname} -g {rg}').get_output_in_json()
        self.assertEqual(len(rules['ipRules']), 0)
        self.assertEqual(len(rules['virtualNetworkRules']), 0)

        # delete the cognitive services account
        ret = self.cmd('az cognitiveservices account delete -n {sname} -g {rg}')
        self.assertEqual(ret.exit_code, 0)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

import os
from azure.cli.testsdk.checkers import JMESPathCheckGreaterThan
from azure.cli.testsdk import ScenarioTest
from azure.cli.testsdk import ResourceGroupPreparer
from .example_steps import step_device_create
from .example_steps import step_device_list
from .example_steps import step_device_list2
from .example_steps import step_device_update
from .example_steps import step_device_show
from .example_steps import step_device_delete
from .. import (
    try_manual,
    raise_if,
    calc_coverage
)


TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))


# Env setup_general
@try_manual
def setup_general(test, rg):
    pass


# Env cleanup_general
@try_manual
def cleanup_general(test, rg):
    pass


# Testcase: general
@try_manual
def call_general(test, rg):
    setup_general(test, rg)
    step_device_create(test, rg, checks=[
        test.check("location", "eastus", case_sensitive=False),
        test.check("sku.name", "Edge", case_sensitive=False),
        test.check("sku.tier", "Standard", case_sensitive=False),
        test.check("name", "{myDevice}", case_sensitive=False),
    ])
    step_device_show(test, rg, checks=[
        test.check("location", "eastus", case_sensitive=False),
        test.check("sku.name", "Edge", case_sensitive=False),
        test.check("sku.tier", "Standard", case_sensitive=False),
        test.check("name", "{myDevice}", case_sensitive=False),
    ])
    step_device_list(test, rg, checks=[
        test.check('length(@)', 1),
    ])
    step_device_list2(test, rg, checks=[
        JMESPathCheckGreaterThan('length(@)', 0),
    ])
    step_device_update(test, rg, checks=[
        test.check("location", "eastus", case_sensitive=False),
        test.check("sku.name", "Edge", case_sensitive=False),
        test.check("sku.tier", "Standard", case_sensitive=False),
        test.check("name", "{myDevice}", case_sensitive=False),
        test.check("tags.Key1", "value1", case_sensitive=False),
        test.check("tags.Key2", "value2", case_sensitive=False),
    ])
    step_device_show(test, rg, checks=[
        test.check("location", "eastus", case_sensitive=False),
        test.check("sku.name", "Edge", case_sensitive=False),
        test.check("sku.tier", "Standard", case_sensitive=False),
        test.check("name", "{myDevice}", case_sensitive=False),
    ])
    step_device_delete(test, rg, checks=[])
    step_device_list(test, rg, checks=[
        test.check('length(@)', 0),
    ])
    cleanup_general(test, rg)


# Test class for general
@try_manual
class DevicegeneralTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='clitestdataboxedge_GroupForEdgeAutomation'[:7], key='rg', parameter_name='rg')
    def test_device_general(self, rg):

        self.kwargs.update({
            'myDevice': 'testedgedevice',
        })

        call_general(self, rg)
        calc_coverage(__file__)
        raise_if()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
from datetime import datetime

from azure_devtools.scenario_tests import AbstractPreparer, SingleValueReplacer

from .base import LiveScenarioTest
from .exceptions import CliTestError
from .reverse_dependency import get_dummy_cli
from .utilities import StorageAccountKeyReplacer, GraphClientPasswordReplacer

KEY_RESOURCE_GROUP = 'rg'
KEY_VIRTUAL_NETWORK = 'vnet'
KEY_VNET_NIC = 'nic'


# This preparer's traffic is not recorded.
# As a result when tests are run in record mode, sdk calls cannot be made to return the prepared resource group.
# Rather the deterministic prepared resource's information should be returned.
class NoTrafficRecordingPreparer(AbstractPreparer):
    from .base import execute as _raw_execute

    def __init__(self, *args, **kwargs):
        super(NoTrafficRecordingPreparer, self).__init__(disable_recording=True, *args, **kwargs)

    def live_only_execute(self, cli_ctx, command, expect_failure=False):
        # call AbstractPreparer.moniker to make resource counts and self.resource_moniker consistent between live and
        # play-back. see SingleValueReplacer.process_request, AbstractPreparer.__call__._preparer_wrapper
        # and ScenarioTest.create_random_name. This is so that when self.create_random_name is called for the
        # first time during live or playback, it would have the same value.
        _ = self.moniker

        try:
            if self.test_class_instance.in_recording:
                return self._raw_execute(cli_ctx, command, expect_failure)
        except AttributeError:
            # A test might not have an in_recording attribute. Run live if this is an instance of LiveScenarioTest
            if isinstance(self.test_class_instance, LiveScenarioTest):
                return self._raw_execute(cli_ctx, command, expect_failure)

        return None


# Resource Group Preparer and its shorthand decorator

class ResourceGroupPreparer(NoTrafficRecordingPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest.rg',
                 parameter_name='resource_group',
                 parameter_name_for_location='resource_group_location', location='westus',
                 dev_setting_name='AZURE_CLI_TEST_DEV_RESOURCE_GROUP_NAME',
                 dev_setting_location='AZURE_CLI_TEST_DEV_RESOURCE_GROUP_LOCATION',
                 random_name_length=75, key='rg'):
        if ' ' in name_prefix:
            raise CliTestError('Error: Space character in resource group name prefix \'%s\'' % name_prefix)
        super(ResourceGroupPreparer, self).__init__(name_prefix, random_name_length)
        self.cli_ctx = get_dummy_cli()
        self.location = location
        self.parameter_name = parameter_name
        self.parameter_name_for_location = parameter_name_for_location
        self.key = key

        self.dev_setting_name = os.environ.get(dev_setting_name, None)
        self.dev_setting_location = os.environ.get(dev_setting_location, location)

    def create_resource(self, name, **kwargs):
        if self.dev_setting_name:
            return {self.parameter_name: self.dev_setting_name,
                    self.parameter_name_for_location: self.dev_setting_location}

        tags = {'product': 'azurecli', 'cause': 'automation', 'date': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')}
        if 'ENV_JOB_NAME' in os.environ:
            tags['job'] = os.environ['ENV_JOB_NAME']
        tags = ' '.join(['{}={}'.format(key, value) for key, value in tags.items()])
        template = 'az group create --location {} --name {} --tag ' + tags
        self.live_only_execute(self.cli_ctx, template.format(self.location, name))

        self.test_class_instance.kwargs[self.key] = name
        return {self.parameter_name: name, self.parameter_name_for_location: self.location}

    def remove_resource(self, name, **kwargs):
        # delete group if test is being recorded and if the group is not a dev rg
        if not self.dev_setting_name:
            self.live_only_execute(self.cli_ctx, 'az group delete --name {} --yes --no-wait'.format(name))


# Storage Account Preparer and its shorthand decorator

# pylint: disable=too-many-instance-attributes
class StorageAccountPreparer(NoTrafficRecordingPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest', sku='Standard_LRS', location='westus', kind='Storage', hns=False,
                 parameter_name='storage_account', resource_group_parameter_name='resource_group', skip_delete=True,
                 dev_setting_name='AZURE_CLI_TEST_DEV_STORAGE_ACCOUNT_NAME', key='sa'):
        super(StorageAccountPreparer, self).__init__(name_prefix, 24)
        self.cli_ctx = get_dummy_cli()
        self.location = location
        self.sku = sku
        self.kind = kind
        self.hns = hns
        self.resource_group_parameter_name = resource_group_parameter_name
        self.skip_delete = skip_delete
        self.parameter_name = parameter_name
        self.key = key
        self.dev_setting_name = os.environ.get(dev_setting_name, None)

    def create_resource(self, name, **kwargs):
        group = self._get_resource_group(**kwargs)

        if not self.dev_setting_name:
            template = 'az storage account create -n {} -g {} -l {} --sku {} --kind {} --https-only '
            if self.hns:
                template += '--hns'
            self.live_only_execute(self.cli_ctx, template.format(
                name, group, self.location, self.sku, self.kind, self.hns))
        else:
            name = self.dev_setting_name

        try:
            account_key = self.live_only_execute(self.cli_ctx,
                                                 'storage account keys list -n {} -g {} --query "[0].value" -otsv'
                                                 .format(name, group)).output
        except AttributeError:  # live only execute returns None if playing from record
            account_key = None

        self.test_class_instance.kwargs[self.key] = name
        return {self.parameter_name: name,
                self.parameter_name + '_info': (name, account_key or StorageAccountKeyReplacer.KEY_REPLACEMENT)}

    def remove_resource(self, name, **kwargs):
        if not self.skip_delete and not self.dev_setting_name:
            group = self._get_resource_group(**kwargs)
            self.live_only_execute(self.cli_ctx, 'az storage account delete -n {} -g {} --yes'.format(name, group))

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create a storage account a resource group is required. Please add ' \
                       'decorator @{} in front of this storage account preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__))


# KeyVault Preparer and its shorthand decorator

# pylint: disable=too-many-instance-attributes
class KeyVaultPreparer(NoTrafficRecordingPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest', sku='standard', location='westus', parameter_name='key_vault',
                 resource_group_parameter_name='resource_group', skip_delete=True,
                 dev_setting_name='AZURE_CLI_TEST_DEV_KEY_VAULT_NAME', key='kv'):
        super(KeyVaultPreparer, self).__init__(name_prefix, 24)
        self.cli_ctx = get_dummy_cli()
        self.location = location
        self.sku = sku
        self.resource_group_parameter_name = resource_group_parameter_name
        self.skip_delete = skip_delete
        self.parameter_name = parameter_name
        self.key = key
        self.dev_setting_name = os.environ.get(dev_setting_name, None)

    def create_resource(self, name, **kwargs):
        if not self.dev_setting_name:
            group = self._get_resource_group(**kwargs)
            template = 'az keyvault create -n {} -g {} -l {} --sku {}'
            self.live_only_execute(self.cli_ctx, template.format(name, group, self.location, self.sku))
            return {self.parameter_name: name}

        self.test_class_instance.kwargs[self.key] = name
        return {self.parameter_name: self.dev_setting_name}

    def remove_resource(self, name, **kwargs):
        if not self.skip_delete and not self.dev_setting_name:
            group = self._get_resource_group(**kwargs)
            self.live_only_execute(self.cli_ctx, 'az keyvault delete -n {} -g {} --yes'.format(name, group))

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create a KeyVault a resource group is required. Please add ' \
                       'decorator @{} in front of this KeyVault preparer.'
            raise CliTestError(template.format(KeyVaultPreparer.__name__))


# Role based access control service principal preparer

# pylint: disable=too-many-instance-attributes
class RoleBasedServicePrincipalPreparer(NoTrafficRecordingPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest',
                 skip_assignment=True, parameter_name='sp_name', parameter_password='sp_password',
                 dev_setting_sp_name='AZURE_CLI_TEST_DEV_SP_NAME',
                 dev_setting_sp_password='AZURE_CLI_TEST_DEV_SP_PASSWORD', key='sp'):
        super(RoleBasedServicePrincipalPreparer, self).__init__(name_prefix, 24)
        self.cli_ctx = get_dummy_cli()
        self.skip_assignment = skip_assignment
        self.result = {}
        self.parameter_name = parameter_name
        self.parameter_password = parameter_password
        self.dev_setting_sp_name = os.environ.get(dev_setting_sp_name, None)
        self.dev_setting_sp_password = os.environ.get(dev_setting_sp_password, None)
        self.key = key

    def create_resource(self, name, **kwargs):
        if not self.dev_setting_sp_name:
            command = 'az ad sp create-for-rbac -n {}{}' \
                .format(name, ' --skip-assignment' if self.skip_assignment else '')

            try:
                self.result = self.live_only_execute(self.cli_ctx, command).get_output_in_json()
            except AttributeError:  # live only execute returns None if playing from record
                pass

            self.test_class_instance.kwargs[self.key] = name
            self.test_class_instance.kwargs['{}_pass'.format(self.key)] = self.parameter_password
            return {self.parameter_name: name,
                    self.parameter_password: self.result.get('password') or GraphClientPasswordReplacer.PWD_REPLACEMENT}

        self.test_class_instance.kwargs[self.key] = self.dev_setting_sp_name
        self.test_class_instance.kwargs['{}_pass'.format(self.key)] = self.dev_setting_sp_password
        return {self.parameter_name: self.dev_setting_sp_name,
                self.parameter_password: self.dev_setting_sp_password}

    def remove_resource(self, name, **kwargs):
        if not self.dev_setting_sp_name:
            self.live_only_execute(self.cli_ctx, 'az ad sp delete --id {}'.format(self.result.get('appId')))


# Managed Application preparer

# pylint: disable=too-many-instance-attributes
class ManagedApplicationPreparer(AbstractPreparer, SingleValueReplacer):
    from .base import execute

    def __init__(self, name_prefix='clitest', parameter_name='aad_client_app_id',
                 parameter_secret='aad_client_app_secret', app_name='app_name',
                 dev_setting_app_name='AZURE_CLI_TEST_DEV_APP_NAME',
                 dev_setting_app_secret='AZURE_CLI_TEST_DEV_APP_SECRET', key='app'):
        super(ManagedApplicationPreparer, self).__init__(name_prefix, 24)
        self.cli_ctx = get_dummy_cli()
        self.parameter_name = parameter_name
        self.parameter_secret = parameter_secret
        self.result = {}
        self.app_name = app_name
        self.dev_setting_app_name = os.environ.get(dev_setting_app_name, None)
        self.dev_setting_app_secret = os.environ.get(dev_setting_app_secret, None)
        self.key = key

    def create_resource(self, name, **kwargs):
        if not self.dev_setting_app_name:
            template = 'az ad app create --display-name {} --key-type Password --password {} --identifier-uris ' \
                       'http://{}'
            self.result = self.execute(self.cli_ctx, template.format(name, name, name)).get_output_in_json()

            self.test_class_instance.kwargs[self.key] = name
            return {self.parameter_name: self.result['appId'], self.parameter_secret: name}
        self.test_class_instance.kwargs[self.key] = name
        return {self.parameter_name: self.dev_setting_app_name,
                self.parameter_secret: self.dev_setting_app_secret}

    def remove_resource(self, name, **kwargs):
        if not self.dev_setting_app_name:
            self.execute(self.cli_ctx, 'az ad app delete --id {}'.format(self.result['appId']))


# pylint: disable=too-many-instance-attributes
class VirtualNetworkPreparer(NoTrafficRecordingPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest.vn', location='westus',
                 parameter_name='virtual_network',
                 resource_group_parameter_name='resource_group',
                 resource_group_key=KEY_RESOURCE_GROUP,
                 dev_setting_name='AZURE_CLI_TEST_DEV_VIRTUAL_NETWORK_NAME',
                 random_name_length=24, key=KEY_VIRTUAL_NETWORK):
        if ' ' in name_prefix:
            raise CliTestError(
                'Error: Space character in name prefix \'%s\'' % name_prefix)
        super(VirtualNetworkPreparer, self).__init__(
            name_prefix, random_name_length)
        self.cli_ctx = get_dummy_cli()
        self.location = location
        self.parameter_name = parameter_name
        self.key = key
        self.resource_group_parameter_name = resource_group_parameter_name
        self.resource_group_key = resource_group_key
        self.dev_setting_name = os.environ.get(dev_setting_name, None)

    def create_resource(self, name, **kwargs):
        if self.dev_setting_name:
            self.test_class_instance.kwargs[self.key] = name
            return {self.parameter_name: self.dev_setting_name, }

        tags = {'product': 'azurecli', 'cause': 'automation',
                'date': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')}
        if 'ENV_JOB_NAME' in os.environ:
            tags['job'] = os.environ['ENV_JOB_NAME']
        tags = ' '.join(['{}={}'.format(key, value)
                         for key, value in tags.items()])
        template = 'az network vnet create --resource-group {} --location {} --name {} --subnet-name default --tag ' + tags
        self.live_only_execute(self.cli_ctx, template.format(self._get_resource_group(**kwargs), self.location, name))

        self.test_class_instance.kwargs[self.key] = name
        return {self.parameter_name: name}

    def remove_resource(self, name, **kwargs):
        if not self.dev_setting_name:
            from msrestazure.azure_exceptions import CloudError
            try:
                self.live_only_execute(
                    self.cli_ctx,
                    'az network vnet delete --name {} --resource-group {}'.format(name, self._get_resource_group(**kwargs)))
            except CloudError:
                # deletion of vnet may fail as service could create subresources like IPConfig. We could rely on the deletion of resource group to delete the vnet.
                pass

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create a VirtualNetwork a resource group is required. Please add ' \
                       'decorator @{} in front of this VirtualNetwork preparer.'
            raise CliTestError(template.format(VirtualNetworkPreparer.__name__))


# pylint: disable=too-many-instance-attributes
class VnetNicPreparer(NoTrafficRecordingPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest.nic',
                 parameter_name='subnet',
                 resource_group_parameter_name=KEY_RESOURCE_GROUP,
                 vnet_parameter_name=KEY_VIRTUAL_NETWORK,
                 dev_setting_name='AZURE_CLI_TEST_DEV_VNET_NIC_NAME',
                 key=KEY_VNET_NIC):
        if ' ' in name_prefix:
            raise CliTestError(
                'Error: Space character in name prefix \'%s\'' % name_prefix)
        super(VnetNicPreparer, self).__init__(name_prefix, 15)
        self.cli_ctx = get_dummy_cli()
        self.parameter_name = parameter_name
        self.key = key
        self.resource_group_parameter_name = resource_group_parameter_name
        self.vnet_parameter_name = vnet_parameter_name
        self.dev_setting_name = os.environ.get(dev_setting_name, None)

    def create_resource(self, name, **kwargs):
        if self.dev_setting_name:
            self.test_class_instance.kwargs[self.key] = name
            return {self.parameter_name: self.dev_setting_name, }

        template = 'az network nic create --resource-group {} --name {} --vnet-name {} --subnet default '
        self.live_only_execute(self.cli_ctx, template.format(
            self._get_resource_group(**kwargs), name, self._get_virtual_network(**kwargs)))

        self.test_class_instance.kwargs[self.key] = name
        return {self.parameter_name: name}

    def remove_resource(self, name, **kwargs):
        if not self.dev_setting_name:
            self.live_only_execute(
                self.cli_ctx,
                'az network nic delete --name {} --resource-group {}'.format(name, self._get_resource_group(**kwargs)))

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create a VirtualNetworkNic a resource group is required. Please add ' \
                       'decorator @{} in front of this VirtualNetworkNic preparer.'
            raise CliTestError(template.format(VnetNicPreparer.__name__))

    def _get_virtual_network(self, **kwargs):
        try:
            return kwargs.get(self.vnet_parameter_name)
        except KeyError:
            template = 'To create a VirtualNetworkNic a virtual network is required. Please add ' \
                       'decorator @{} in front of this VirtualNetworkNic preparer.'
            raise CliTestError(template.format(VnetNicPreparer.__name__))

# Utility


def is_preparer_func(fn):
    return getattr(fn, '__is_preparer', False)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import base64
import unittest
import tempfile
import logging
import logging.handlers

from azure.cli.telemetry.components.telemetry_logging import get_logger, LOGGER_NAME, config_logging_for_upload
from azure.cli.telemetry.const import TELEMETRY_LOG_NAME, TELEMETRY_LOG_DIR


class TestTelemetryLogging(unittest.TestCase):
    def test_create_logger(self):
        random_name = base64.b64encode(os.urandom(8)).decode('utf-8')
        logger = get_logger(random_name)

        self.assertEqual('{}.{}'.format(LOGGER_NAME, random_name), logger.name)

        logger.debug('come from {}'.format(random_name))


class TestTelemetryUploadLogging(unittest.TestCase):
    def setUp(self):
        self._original_handlers = logging.root.handlers
        self._original_level = logging.root.level
        del logging.root.handlers[:]

    def tearDown(self):
        for handler in logging.root.handlers:
            if isinstance(handler, logging.handlers.RotatingFileHandler):
                if handler.stream:
                    handler.stream.close()

        del logging.root.handlers[:]
        logging.root.setLevel(self._original_level)
        for handler in self._original_handlers:
            logging.root.addHandler(handler)

    def test_config_logging_for_upload_process(self):
        temp_dir = tempfile.mkdtemp()
        random_name = base64.b64encode(os.urandom(8)).decode('utf-8')

        config_logging_for_upload(temp_dir)

        logger = get_logger(random_name)
        logger.info('come from {}'.format(random_name))

        log_file = os.path.join(temp_dir, TELEMETRY_LOG_DIR, TELEMETRY_LOG_NAME)
        self.assertTrue(os.path.exists(log_file))

        with open(log_file, mode='r') as fq:
            content = fq.read().strip('\n')
            self.assertTrue(content.endswith(random_name),
                            'Log content {} does not contain {}'.format(content, random_name))

    def test_config_logging_for_upload_process_nonexist(self):
        temp_dir = tempfile.mktemp()
        self.assertFalse(os.path.isdir(temp_dir))

        random_name = base64.b64encode(os.urandom(8)).decode('utf-8')

        config_logging_for_upload(temp_dir)

        logger = get_logger(random_name)
        logger.info('come from {}'.format(random_name))

        log_file = os.path.join(temp_dir, TELEMETRY_LOG_DIR, TELEMETRY_LOG_NAME)
        self.assertTrue(os.path.exists(log_file))

        with open(log_file, mode='r') as fq:
            content = fq.read().strip('\n')
            self.assertTrue(content.endswith(random_name),
                            'Log content {} does not contain {}'.format(content, random_name))


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer,
                               JMESPathCheck, NoneCheck, api_version_constraint)
from azure.cli.core.profiles import ResourceType
from ..storage_test_util import StorageScenarioMixin


@api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2016-12-01')
class StorageTableScenarioTests(StorageScenarioMixin, ScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer(sku='Standard_RAGRS')
    def test_storage_table_main_scenario(self, resource_group, storage_account):
        account_info = self.get_account_info(resource_group, storage_account)
        table_name = self.create_random_name('table', 24)

        self.storage_cmd('storage table create -n {} --fail-on-exist', account_info, table_name) \
            .assert_with_checks(JMESPathCheck('created', True))
        self.storage_cmd('storage table exists -n {}', account_info, table_name) \
            .assert_with_checks(JMESPathCheck('exists', True))

        self.assertIn(table_name, self.storage_cmd('storage table list --query "[].name"',
                                                   account_info).get_output_in_json())
        from datetime import datetime, timedelta
        expiry = (datetime.utcnow() + timedelta(hours=1)).strftime('%Y-%m-%dT%H:%MZ')
        sas = self.storage_cmd('storage table generate-sas -n {} --permissions r --expiry {}',
                               account_info, table_name, expiry).output
        self.assertIn('sig=', sas)

        self.verify_entity_operations(account_info, table_name)
        self.verify_table_acl_operations(account_info, table_name)

        # verify delete operation
        self.storage_cmd('storage table delete --name {} --fail-not-exist',
                         account_info, table_name) \
            .assert_with_checks(JMESPathCheck('deleted', True))

        self.storage_cmd('storage table exists -n {}', account_info, table_name) \
            .assert_with_checks(JMESPathCheck('exists', False))

        # status may not be available immediately after the storage account is created in live testing. so retry a few
        # times
        table_status = self.storage_cmd('storage table stats', account_info).get_output_in_json()
        self.assertIn(table_status['geoReplication']['status'], ('live', 'unavailable'))

    def verify_entity_operations(self, account_info, table_name):
        self.storage_cmd(
            'storage entity insert -t {} -e rowkey=001 partitionkey=001 name=test value=something '
            'binaryProperty=AAECAwQF binaryProperty@odata.type=Edm.Binary',
            account_info, table_name)
        self.storage_cmd('storage entity show -t {} --row-key 001 --partition-key 001',
                         account_info, table_name) \
            .assert_with_checks(JMESPathCheck('name', 'test'),
                                JMESPathCheck('value', 'something'),
                                JMESPathCheck('binaryProperty.value', 'AAECAwQF'))
        self.storage_cmd(
            'storage entity show -t {} --row-key 001 --partition-key 001 --select name',
            account_info, table_name).assert_with_checks(JMESPathCheck('name', 'test'),
                                                         JMESPathCheck('value', None),
                                                         JMESPathCheck('binaryProperty.value', None))
        self.storage_cmd('storage entity merge -t {} -e rowkey=001 partitionkey=001 name=test value=newval',
                         account_info, table_name)
        self.storage_cmd('storage entity show -t {} --row-key 001 --partition-key 001',
                         account_info, table_name) \
            .assert_with_checks(JMESPathCheck('name', 'test'),
                                JMESPathCheck('value', 'newval'),
                                JMESPathCheck('binaryProperty.value', 'AAECAwQF'))

        self.storage_cmd('storage entity replace -t {} -e rowkey=001 partitionkey=001 cat=hat',
                         account_info, table_name)
        self.storage_cmd('storage entity show -t {} --row-key 001 --partition-key 001',
                         account_info, table_name) \
            .assert_with_checks(JMESPathCheck('cat', 'hat'), JMESPathCheck('name', None),
                                JMESPathCheck('value', None), JMESPathCheck('binaryProperty.value', None))

        self.storage_cmd('storage entity delete -t {} --row-key 001 --partition-key 001',
                         account_info, table_name)
        self.storage_cmd_negative('storage entity show -t {} --row-key 001 --partition-key 001',
                                  account_info, table_name)
        self.storage_cmd('storage entity insert -t {} -e rowkey=001 partitionkey=001 name=test value=something '
                         'binaryProperty=AAECAwQF binaryProperty@odata.type=Edm.Binary',
                         account_info, table_name)
        self.storage_cmd('storage entity insert -t {} -e rowkey=002 partitionkey=002 name=test2 value=something2',
                         account_info, table_name)
        result = self.storage_cmd('storage entity query -t {} --num-results 1',
                                  account_info, table_name).get_output_in_json()
        marker = result.get('nextMarker')
        self.storage_cmd('storage entity query -t {} --marker nextpartitionkey={} nextrowkey={}', account_info,
                         table_name, marker.get('nextpartitionkey'), marker.get('nextrowkey')).assert_with_checks(
                             JMESPathCheck('length(items)', 1))

    def verify_table_acl_operations(self, account_info, table_name):
        self.storage_cmd('storage table policy list -t {}', account_info,
                         table_name).assert_with_checks(NoneCheck())
        self.storage_cmd('storage table policy create -t {} -n test1 --permission a', account_info,
                         table_name)
        self.storage_cmd('storage table policy create -t {} -n test2 --start 2016-01-01T00:00Z',
                         account_info, table_name)
        self.storage_cmd('storage table policy create -t {} -n test3 --expiry 2018-01-01T00:00Z',
                         account_info, table_name)
        self.storage_cmd('storage table policy create -t {} -n test4 --permission raud --start '
                         '2016-01-01T00:00Z --expiry 2016-05-01T00:00Z', account_info, table_name)

        acl = self.storage_cmd('storage table policy list -t {}', account_info,
                               table_name).get_output_in_json().keys()
        self.assertSetEqual(set(acl), set(['test1', 'test2', 'test3', 'test4']))

        self.storage_cmd('storage table policy show -t {} -n test1', account_info,
                         table_name).assert_with_checks(JMESPathCheck('permission', 'a'))
        self.storage_cmd('storage table policy show -t {} -n test2', account_info,
                         table_name) \
            .assert_with_checks(JMESPathCheck('start', '2016-01-01T00:00:00+00:00'))
        self.storage_cmd('storage table policy show -t {} -n test3', account_info, table_name) \
            .assert_with_checks(JMESPathCheck('expiry', '2018-01-01T00:00:00+00:00'))
        self.storage_cmd('storage table policy show -t {} -n test4', account_info, table_name) \
            .assert_with_checks(JMESPathCheck('start', '2016-01-01T00:00:00+00:00'),
                                JMESPathCheck('expiry', '2016-05-01T00:00:00+00:00'),
                                JMESPathCheck('permission', 'raud'))

        self.storage_cmd('storage table policy update -t {} -n test1 --permission au', account_info,
                         table_name)
        self.storage_cmd('storage table policy show -t {} -n test1', account_info, table_name) \
            .assert_with_checks(JMESPathCheck('permission', 'au'))
        self.storage_cmd('storage table policy delete -t {} -n test1', account_info, table_name)

        acl = self.storage_cmd('storage table policy list -t {}', account_info,
                               table_name).get_output_in_json().keys()
        self.assertSetEqual(set(acl), set(['test2', 'test3', 'test4']))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import argparse
import json
from mock import patch
from os.path import expanduser
from docutils import nodes
from docutils.statemachine import ViewList
try:
    # Deprecated in 1.6 and removed in 1.7
    from sphinx.util.compat import Directive
except ImportError:
    from docutils.parsers.rst import Directive  # pylint: disable=import-error
from sphinx.util.nodes import nested_parse_with_titles


from azure.cli.core import MainCommandsLoader, AzCli
from azure.cli.core.commands import AzCliCommandInvoker
from azure.cli.core.parser import AzCliCommandParser
from azure.cli.core.file_util import create_invoker_and_load_cmds_and_args, get_all_help
from azure.cli.core._help import AzCliHelp, CliCommandHelpFile, ArgumentGroupRegistry

USER_HOME = expanduser('~')


class AzHelpGenDirective(Directive):
    def make_rst(self):
        INDENT = '   '
        DOUBLEINDENT = INDENT * 2

        az_cli = AzCli(cli_name='az',
               commands_loader_cls=MainCommandsLoader,
               invocation_cls=AzCliCommandInvoker,
               parser_cls=AzCliCommandParser,
               help_cls=AzCliHelp)
        with patch('getpass.getuser', return_value='your_system_user_login_name'):
            create_invoker_and_load_cmds_and_args(az_cli)
        help_files = get_all_help(az_cli)

        doc_source_map = _load_doc_source_map()

        for help_file in help_files:
            is_command = isinstance(help_file, CliCommandHelpFile)
            yield '.. cli{}:: {}'.format('command' if is_command else 'group', help_file.command if help_file.command else 'az') #it is top level group az if command is empty
            yield ''
            yield '{}:summary: {}'.format(INDENT, help_file.short_summary)
            yield '{}:description: {}'.format(INDENT, help_file.long_summary)
            if help_file.deprecate_info:
                yield '{}:deprecated: {}'.format(INDENT, help_file.deprecate_info._get_message(help_file.deprecate_info))
            if not is_command:
                top_group_name = help_file.command.split()[0] if help_file.command else 'az' 
                yield '{}:docsource: {}'.format(INDENT, doc_source_map[top_group_name] if top_group_name in doc_source_map else '')
            else:
                top_command_name = help_file.command.split()[0] if help_file.command else ''
                if top_command_name in doc_source_map:
                    yield '{}:docsource: {}'.format(INDENT, doc_source_map[top_command_name])
            yield ''

            if is_command and help_file.parameters:
               group_registry = ArgumentGroupRegistry(
                  [p.group_name for p in help_file.parameters if p.group_name]) 

               for arg in sorted(help_file.parameters,
                                key=lambda p: group_registry.get_group_priority(p.group_name)
                                + str(not p.required) + p.name):
                    yield '{}.. cliarg:: {}'.format(INDENT, arg.name)
                    yield ''
                    yield '{}:required: {}'.format(DOUBLEINDENT, arg.required)
                    if arg.deprecate_info:
                        yield '{}:deprecated: {}'.format(DOUBLEINDENT, arg.deprecate_info._get_message(arg.deprecate_info))
                    short_summary = arg.short_summary or ''
                    possible_values_index = short_summary.find(' Possible values include')
                    short_summary = short_summary[0:possible_values_index
                                                    if possible_values_index >= 0 else len(short_summary)]
                    short_summary = short_summary.strip()
                    yield '{}:summary: {}'.format(DOUBLEINDENT, short_summary)
                    yield '{}:description: {}'.format(DOUBLEINDENT, arg.long_summary)
                    if arg.choices:
                        yield '{}:values: {}'.format(DOUBLEINDENT, ', '.join(sorted([str(x) for x in arg.choices])))
                    if arg.default and arg.default != argparse.SUPPRESS:
                        try:
                            if arg.default.startswith(USER_HOME):
                                arg.default = arg.default.replace(USER_HOME, '~').replace('\\', '/')
                        except Exception:
                            pass
                        try:
                            arg.default = arg.default.replace("\\", "\\\\")
                        except Exception:
                            pass
                        yield '{}:default: {}'.format(DOUBLEINDENT, arg.default)
                    if arg.value_sources:
                        yield '{}:source: {}'.format(DOUBLEINDENT, ', '.join(_get_populator_commands(arg)))
                    yield ''
            yield ''
            if len(help_file.examples) > 0:
               for e in help_file.examples:
                  yield '{}.. cliexample:: {}'.format(INDENT, e.short_summary)
                  yield ''
                  yield DOUBLEINDENT + e.command.replace("\\", "\\\\")
                  yield ''

    def run(self):
        node = nodes.section()
        node.document = self.state.document
        result = ViewList()
        for line in self.make_rst():
            result.append(line, '<azhelpgen>')

        nested_parse_with_titles(self.state, result, node)
        return node.children

def setup(app):
    app.add_directive('azhelpgen', AzHelpGenDirective)


def _store_parsers(parser, parser_keys, parser_values, sub_parser_keys, sub_parser_values):
    for s in parser.subparsers.values():
        parser_keys.append(_get_parser_name(s))
        parser_values.append(s)
        if _is_group(s):
            for c in s.choices.values():
                sub_parser_keys.append(_get_parser_name(c))
                sub_parser_values.append(c)
                _store_parsers(c, parser_keys, parser_values, sub_parser_keys, sub_parser_values)

def _load_doc_source_map():
    with open('azhelpgen/doc_source_map.json') as open_file:
        return json.load(open_file)

def _is_group(parser):
    return getattr(parser, '_subparsers', None) is not None \
        or getattr(parser, 'choices', None) is not None

def _get_parser_name(s):
    return (s._prog_prefix if hasattr(s, '_prog_prefix') else s.prog)[3:]


def _get_populator_commands(param):
    commands = []
    for value_source in param.value_sources:
        try:
            commands.append(value_source["link"]["command"])
        except KeyError:
            continue
    return commands



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer
from knack.util import CLIError

POOL_DEFAULT = "--service-level Premium --size 4"
POOL_DEFAULT_TOO_SMALL = "--service-level 'Premium' --size 3"
POOL_DEFAULT_STRING_SIZE = "--service-level 'Premium' --size a"
LOCATION = "westus2"

# No tidy up of tests required. The resource group is automatically removed


class AzureNetAppFilesPoolServiceScenarioTest(ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_pool_')
    def test_create_delete_pool(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        tags = "Tag1=Value1 Tag2=Value2"

        self.cmd("az netappfiles account create --resource-group {rg} --account-name '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()
        pool = self.cmd("az netappfiles pool create --resource-group {rg} --account-name %s --pool-name %s -l %s %s --tags %s" % (account_name, pool_name, LOCATION, POOL_DEFAULT, tags)).get_output_in_json()
        assert pool['name'] == account_name + '/' + pool_name
        assert pool['tags']['Tag1'] == 'Value1'
        assert pool['tags']['Tag2'] == 'Value2'

        pool_list = self.cmd("netappfiles pool list --resource-group {rg} --account-name %s" % account_name).get_output_in_json()
        assert len(pool_list) == 1

        self.cmd("az netappfiles pool delete --resource-group {rg} --account-name '%s' --pool-name '%s'" % (account_name, pool_name))
        pool_list = self.cmd("netappfiles pool list --resource-group {rg} --account-name %s" % account_name).get_output_in_json()
        assert len(pool_list) == 0

        # and again with short forms and also unquoted
        pool = self.cmd("az netappfiles pool create -g {rg} -a %s -p %s -l %s --service-level 'Premium' --size 4 --tags %s" % (account_name, pool_name, LOCATION, tags)).get_output_in_json()
        assert pool['name'] == account_name + '/' + pool_name
        assert pool['tags']['Tag1'] == 'Value1'
        assert pool['tags']['Tag2'] == 'Value2'
        assert pool['qosType'] == 'Auto'

        self.cmd("az netappfiles pool delete --resource-group {rg} -a %s -p %s" % (account_name, pool_name))
        pool_list = self.cmd("netappfiles pool list --resource-group {rg} -a %s" % account_name).get_output_in_json()
        assert len(pool_list) == 0

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_pool_')
    def test_create_pool_too_small(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)

        self.cmd("az netappfiles account create --resource-group {rg} --account-name '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()
        try:
            self.cmd("az netappfiles pool create --resource-group {rg} --account-name %s --pool-name %s -l %s %s " % (account_name, pool_name, LOCATION, POOL_DEFAULT_TOO_SMALL)).get_output_in_json()
        except Exception as ex:
            assert isinstance(ex, CLIError)

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_pool_')
    def test_create_pool_string_size(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)

        self.cmd("az netappfiles account create --resource-group {rg} --account-name '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()
        try:
            self.cmd("az netappfiles pool create --resource-group {rg} --account-name %s --pool-name %s -l %s %s " % (account_name, pool_name, LOCATION, POOL_DEFAULT_STRING_SIZE)).get_output_in_json()
        except Exception as ex:
            assert isinstance(ex, CLIError)

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_pool_')
    def test_list_pools(self):
        account_name = self.create_random_name(prefix='cli', length=24)
        pools = [self.create_random_name(prefix='cli', length=24), self.create_random_name(prefix='cli', length=24)]
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()

        for pool_name in pools:
            self.cmd("az netappfiles pool create -g {rg} -a '%s' -p '%s' -l %s %s --tags Tag1=Value1" % (account_name, pool_name, LOCATION, POOL_DEFAULT)).get_output_in_json()

        pool_list = self.cmd("netappfiles pool list -g {rg} -a '%s'" % account_name).get_output_in_json()
        assert len(pool_list) == 2

        for pool_name in pools:
            self.cmd("az netappfiles pool delete -g {rg} -a %s -p %s" % (account_name, pool_name))
        pool_list = self.cmd("netappfiles pool list --resource-group {rg} -a '%s'" % account_name).get_output_in_json()
        assert len(pool_list) == 0

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_pool_')
    def test_get_pool_by_name(self):
        account_name = self.create_random_name(prefix='cli', length=24)
        pool_name = self.create_random_name(prefix='cli', length=24)
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()
        self.cmd("az netappfiles pool create -g {rg} -a %s -p %s -l %s %s" % (account_name, pool_name, LOCATION, POOL_DEFAULT)).get_output_in_json()

        pool = self.cmd("az netappfiles pool show --resource-group {rg} -a %s -p %s" % (account_name, pool_name)).get_output_in_json()
        assert pool['name'] == account_name + '/' + pool_name
        pool_from_id = self.cmd("az netappfiles pool show --ids %s" % pool['id']).get_output_in_json()
        assert pool_from_id['name'] == account_name + '/' + pool_name

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_pool_')
    def test_update_pool(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        tag = "Tag1=Value1"

        self.cmd("az netappfiles account create -g {rg} -a %s -l %s" % (account_name, LOCATION)).get_output_in_json()

        pool = self.cmd("az netappfiles pool create -g {rg} -a %s -p %s -l %s %s" % (account_name, pool_name, LOCATION, POOL_DEFAULT)).get_output_in_json()

        assert pool['name'] == account_name + '/' + pool_name
        pool = self.cmd("az netappfiles pool update --resource-group {rg} -a %s -p %s --tags %s --qos-type %s" % (account_name, pool_name, tag, "Manual")).get_output_in_json()
        assert pool['name'] == account_name + '/' + pool_name
        assert pool['qosType'] == "Manual"
        assert pool['tags']['Tag1'] == 'Value1'



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from enum import Enum
import importlib
import inspect
import logging
from pathlib import Path
import pkgutil
import shutil
import sys
import tempfile

from msrest.serialization import Model
from msrest.paging import Paged

_LOGGER = logging.getLogger(__name__)

copyright_header = b"""# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

"""

header = copyright_header + b"""from msrest.serialization import Model
from msrest.exceptions import HttpOperationError
"""

track2_header = copyright_header + b"""import datetime
import msrest
import msrest.serialization
from typing import Dict, List, Optional, Union
from msrest.exceptions import HttpOperationError
"""

paging_header = copyright_header + b"""from msrest.paging import Paged
"""

init_file = """
try:
    from .{} import *
except (SyntaxError, ImportError):
    from .{} import *
from .{} import *
"""


def parse_input(input_parameter):
    """From a syntax like package_name#submodule, build a package name
    and complete module name.
    """
    split_package_name = input_parameter.split('#')
    package_name = split_package_name[0]
    module_name = package_name.replace("-", ".")
    if len(split_package_name) >= 2:
        module_name = ".".join([module_name, split_package_name[1]])
    return package_name, module_name


def solve_mro(models, track2=False):
    for models_module in models:
        models_path = models_module.__path__[0]
        _LOGGER.info("Working on %s", models_path)

        if not track2 and Path(models_path, "models_py3.py").exists():
            _LOGGER.info("Skipping since already patched")
            return

        # Build the new files in a temp folder
        with tempfile.TemporaryDirectory() as temp_folder:
            final_models_path = Path(temp_folder, "models")
            final_models_path.mkdir()
            solve_one_model(models_module, final_models_path, track2=track2)

            # Switch the files
            shutil.rmtree(models_path)
            shutil.move(final_models_path, models_path)


def solve_one_model(models_module, output_folder, track2=False):
    """Will build the compacted models in the output_folder"""

    models_classes = [
        (len(model_class.__mro__), inspect.getfile(model_class), model_class) for model_name, model_class in
        vars(models_module).items()
        if model_name[0].isupper() and Model in model_class.__mro__
    ]
    # Only sort based on the first element in the tuple
    models_classes.sort(key=lambda x: x[0])

    py2_models_classes = [
        (len_mro, path.replace("_py3.py", ".py"), None)
        for len_mro, path, _ in models_classes
    ]

    paged_models_classes = [
        (inspect.getfile(model_class), model_class) for model_name, model_class in vars(models_module).items()
        if model_name[0].isupper() and Paged in model_class.__mro__
    ]

    enum_models_classes = [
        (inspect.getfile(model_class), model_class) for model_name, model_class in vars(models_module).items()
        if model_name[0].isupper() and Enum in model_class.__mro__
    ]
    if enum_models_classes:
        enum_file = Path(enum_models_classes[0][0])
        shutil.copyfile(enum_file, Path(output_folder, enum_file.name))
        enum_file_module_name = enum_file.with_suffix('').name
    else:
        enum_file_module_name = None

    write_model_file(Path(output_folder, "models_py3.py"), models_classes, track2=track2)
    write_model_file(Path(output_folder, "models.py"), py2_models_classes, track2=track2)
    write_paging_file(Path(output_folder, "paged_models.py"), paged_models_classes)
    write_init(
        Path(output_folder, "__init__.py"),
        "models_py3",
        "models",
        "paged_models",
        enum_file_module_name
    )


def write_model_file(output_file_path, classes_to_write, track2=False):
    with open(output_file_path, "bw") as write_fd:
        if track2:
            write_fd.write(track2_header)
        else:
            write_fd.write(header)

        for model in classes_to_write:
            _, model_file_path, _ = model

            with open(model_file_path, "rb") as read_fd:
                lines = read_fd.readlines()
                # Skip until it's "class XXXX"
                while lines:
                    if lines[0].startswith(b"class "):
                        break
                    lines.pop(0)
                else:
                    raise ValueError("Never found any class definition!")
                # Now I keep everything
                write_fd.write(b'\n')
                write_fd.write(b'\n')
                write_fd.writelines(lines)


def write_paging_file(output_file_path, classes_to_write):
    with open(output_file_path, "bw") as write_fd:
        write_fd.write(paging_header)

        for model in classes_to_write:
            model_file_path, _ = model

            with open(model_file_path, "rb") as read_fd:
                # Skip the first 15 lines (based on Autorest deterministic behavior)
                # If we want this less random, look for the first line starts with "class"
                lines = read_fd.readlines()[14:]
                write_fd.write(b'\n')
                write_fd.write(b'\n')
                write_fd.writelines(lines)


def write_init(output_file_path, model_file_name, model_file_name_py2, paging_file_name, enum_file_name):
    with open(output_file_path, "bw") as write_fd:
        write_fd.write(copyright_header)

        write_fd.write(init_file.format(
            model_file_name,
            model_file_name_py2,
            paging_file_name,
        ).encode('utf8'))
        if enum_file_name:
            write_fd.write(
                "from .{} import *".format(enum_file_name).encode('utf8')
            )


def find_models_to_change(module_name):
    """Will figure out if the package is a multi-api one,
    and understand what to generate.
    """
    main_module = importlib.import_module(module_name)
    try:
        models_module = main_module.models
        models_module.__path__
        # It didn't fail, that's a single API package
        return [models_module]
    except AttributeError:
        # This means I loaded the fake module "models"
        # and it's multi-api, load all models
        return [
            importlib.import_module('.' + label + '.models', main_module.__name__)
            for (_, label, ispkg) in pkgutil.iter_modules(main_module.__path__)
            if ispkg and label != 'aio'
        ]


def find_autorest_generated_folder(module_prefix="azure.mgmt"):
    """Find all Autorest generated code in that module prefix.

    This actually looks for a "models" package only. We could be smarter if necessary.
    """
    _LOGGER.info("Looking for Autorest generated package in %s", module_prefix)
    result = []
    prefix_module = importlib.import_module(module_prefix)
    for _, sub_package, ispkg in pkgutil.iter_modules(prefix_module.__path__, module_prefix + "."):
        try:
            _LOGGER.debug("Try %s", sub_package)
            importlib.import_module(".models", sub_package)
            # If not exception, we found it
            _LOGGER.info("Found %s", sub_package)
            result.append(sub_package)
        except ModuleNotFoundError:
            # No model, might dig deeper
            if ispkg:
                result += find_autorest_generated_folder(sub_package)
    return result


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)

    track2_packages = [
        'azure.mgmt.keyvault',
        'azure.mgmt.storage',
        'azure.mgmt.compute',
        'azure.mgmt.network'
    ]
    prefix = sys.argv[1] if len(sys.argv) >= 2 else "azure.mgmt"
    for autorest_package in find_autorest_generated_folder(prefix):
        models = find_models_to_change(autorest_package)
        track2 = False
        for track2_pkg in track2_packages:
            if autorest_package.startswith(track2_pkg):
                track2 = True
                break
        solve_mro(models, track2=track2)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer, live_only
import mock
from msrestazure.tools import resource_id


class MonitorCloneVMScenarios(ScenarioTest):
    @ResourceGroupPreparer(location='eastus')
    def test_monitor_clone_vm_metric_alerts_scenario(self, resource_group):
        self.kwargs.update({
            'alert': 'alert1',
            'vm1': 'vm1',
            'vm2': 'vm2',
            'vm3': 'vm3',
            'ag1': 'ag1',
            'rg': resource_group
        })

        vm1_json = self.cmd('vm create -g {rg} -n {vm1} --image UbuntuLTS --admin-password TestPassword11!! '
                            '--admin-username testadmin --authentication-type password').get_output_in_json()
        vm2_json = self.cmd('vm create -g {rg} -n {vm2} --image UbuntuLTS --admin-password TestPassword11!! '
                            '--admin-username testadmin --authentication-type password').get_output_in_json()
        vm3_json = self.cmd('vm create -g {rg} -n {vm3} --image UbuntuLTS --admin-password TestPassword11!! '
                            '--admin-username testadmin --authentication-type password').get_output_in_json()
        self.kwargs.update({
            'vm1_id': vm1_json['id'],
            'vm2_id': vm2_json['id'],
            'vm3_id': vm3_json['id']
        })
        self.cmd('monitor action-group create -g {rg} -n {ag1}')
        self.cmd('monitor metrics alert create -g {rg} -n {alert} --scopes {vm1_id} {vm2_id} --action {ag1} --condition "avg Percentage CPU > 90" --description "High CPU"', checks=[
            self.check('description', 'High CPU'),
            self.check('severity', 2),
            self.check('autoMitigate', None),
            self.check('windowSize', '0:05:00'),
            self.check('evaluationFrequency', '0:01:00'),
            self.check('length(scopes)', 2)
        ])
        with mock.patch('azure.cli.command_modules.monitor.operations.monitor_clone_util.gen_guid', side_effect=self.create_guid):
            self.cmd('monitor clone --source-resource {vm1_id} --target-resource {vm3_id}', checks=[
                self.check('metricsAlert[0].description', 'High CPU'),
                self.check('metricsAlert[0].severity', 2),
                self.check('metricsAlert[0].autoMitigate', None),
                self.check('metricsAlert[0].windowSize', '0:05:00'),
                self.check('metricsAlert[0].evaluationFrequency', '0:01:00'),
                self.check('length(metricsAlert[0].scopes)', 3)
            ])


class MonitorCloneStorageAccountScenarios(ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_test_metric_alert_clone')
    @StorageAccountPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_2')
    def test_monitor_clone_storage_metric_alerts_scenario(self, resource_group, storage_account, storage_account_2):
        self.kwargs.update({
            'alert': 'alert1',
            'sa': storage_account,
            'ag1': 'ag1',
            'sub': self.get_subscription_id(),
            'sa_id': resource_id(
                resource_group=resource_group,
                subscription=self.get_subscription_id(),
                name=storage_account,
                namespace='Microsoft.Storage',
                type='storageAccounts'),
            'sa_id_2': resource_id(
                resource_group=resource_group,
                subscription=self.get_subscription_id(),
                name=storage_account_2,
                namespace='Microsoft.Storage',
                type='storageAccounts'),
        })

        self.cmd('monitor action-group create -g {rg} -n {ag1}')
        self.cmd('monitor metrics alert create -g {rg} -n {alert} --scopes {sa_id} --action {ag1} --description "Test" --condition "total transactions > 5 where ResponseType includes Success and ApiName includes GetBlob" --condition "avg SuccessE2ELatency > 250 where ApiName includes GetBlob"', checks=[
            self.check('description', 'Test'),
            self.check('severity', 2),
            self.check('autoMitigate', None),
            self.check('windowSize', '0:05:00'),
            self.check('evaluationFrequency', '0:01:00'),
            self.check('length(criteria.allOf)', 2),
            self.check('length(criteria.allOf[0].dimensions)', 2),
            self.check('length(criteria.allOf[1].dimensions)', 1)
        ])

        with mock.patch('azure.cli.command_modules.monitor.operations.monitor_clone_util.gen_guid', side_effect=self.create_guid):
            self.cmd('monitor clone --source-resource {sa_id} --target-resource {sa_id_2}', checks=[
                self.check('metricsAlert[0].description', 'Test'),
                self.check('metricsAlert[0].severity', 2),
                self.check('metricsAlert[0].autoMitigate', None),
                self.check('metricsAlert[0].windowSize', '0:05:00'),
                self.check('metricsAlert[0].evaluationFrequency', '0:01:00'),
                self.check('length(metricsAlert[0].criteria.allOf)', 2),
                self.check('length(metricsAlert[0].criteria.allOf[0].dimensions)', 2),
                self.check('length(metricsAlert[0].criteria.allOf[1].dimensions)', 1),
            ])


class MonitorCloneStorageAccountAlwaysScenarios(ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_test_metric_alert_clone')
    @StorageAccountPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_2')
    def test_monitor_clone_storage_metric_alerts_always_scenario(self, resource_group, storage_account, storage_account_2):
        self.kwargs.update({
            'alert': 'alert1',
            'sa': storage_account,
            'ag1': 'ag1',
            'sub': self.get_subscription_id(),
            'sa_id': resource_id(
                resource_group=resource_group,
                subscription=self.get_subscription_id(),
                name=storage_account,
                namespace='Microsoft.Storage',
                type='storageAccounts'),
            'sa_id_2': resource_id(
                resource_group=resource_group,
                subscription=self.get_subscription_id(),
                name=storage_account_2,
                namespace='Microsoft.Storage',
                type='storageAccounts'),
        })

        self.cmd('monitor action-group create -g {rg} -n {ag1}')
        self.cmd('monitor metrics alert create -g {rg} -n {alert} --scopes {sa_id} --action {ag1} --description "Test" --condition "total transactions > 5 where ResponseType includes Success and ApiName includes GetBlob" --condition "avg SuccessE2ELatency > 250 where ApiName includes GetBlob"', checks=[
            self.check('description', 'Test'),
            self.check('severity', 2),
            self.check('autoMitigate', None),
            self.check('windowSize', '0:05:00'),
            self.check('evaluationFrequency', '0:01:00'),
            self.check('length(criteria.allOf)', 2),
            self.check('length(criteria.allOf[0].dimensions)', 2),
            self.check('length(criteria.allOf[1].dimensions)', 1)
        ])

        with mock.patch('azure.cli.command_modules.monitor.operations.monitor_clone_util.gen_guid', side_effect=self.create_guid):
            self.cmd('monitor clone --source-resource {sa_id} --target-resource {sa_id_2} --always-clone', checks=[
                self.check('metricsAlert[0].description', 'Test'),
                self.check('metricsAlert[0].severity', 2),
                self.check('metricsAlert[0].autoMitigate', None),
                self.check('metricsAlert[0].windowSize', '0:05:00'),
                self.check('metricsAlert[0].evaluationFrequency', '0:01:00'),
                self.check('length(metricsAlert[0].criteria.allOf)', 2),
                self.check('length(metricsAlert[0].criteria.allOf[0].dimensions)', 2),
                self.check('length(metricsAlert[0].criteria.allOf[1].dimensions)', 1),
            ])


class MonitorClonePublicIpScenarios(ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_test_metric_alert_clone')
    def test_monitor_clone_public_ip_metric_alerts_scenario(self, resource_group):
        self.kwargs.update({
            'alert': 'alert1',
            'alert2': 'alert2',
            'ag1': 'ag1',
            'sub': self.get_subscription_id(),
            'rg': resource_group,
            'ip1': 'ip1',
            'ip2': 'ip2'
        })

        ip1_json = self.cmd('network public-ip create -g {rg} -n {ip1}').get_output_in_json()
        ip2_json = self.cmd('network public-ip create -g {rg} -n {ip2}').get_output_in_json()
        self.kwargs.update({
            'ip1_id': ip1_json['publicIp']['id'],
            'ip2_id': ip2_json['publicIp']['id']
        })

        self.cmd('monitor action-group create -g {rg} -n {ag1}')
        self.cmd('monitor metrics alert create -g {rg} -n {alert} --scopes {ip1_id} --action {ag1} --description "Test" --condition "total TCPBytesForwardedDDoS > 5"', checks=[
            self.check('description', 'Test'),
            self.check('severity', 2),
            self.check('windowSize', '0:05:00'),
            self.check('evaluationFrequency', '0:01:00'),
            self.check('length(scopes)', 1)
        ])

        self.cmd('monitor metrics alert create -g {rg} -n {alert2} --scopes {ip1_id} --action {ag1} --description "Test2" --condition "max TCPBytesForwardedDDoS > 5"', checks=[
            self.check('description', 'Test2'),
            self.check('severity', 2),
            self.check('windowSize', '0:05:00'),
            self.check('evaluationFrequency', '0:01:00'),
            self.check('length(scopes)', 1)
        ])

        with mock.patch('azure.cli.command_modules.monitor.operations.monitor_clone_util.gen_guid', side_effect=self.create_guid):
            self.cmd('monitor clone --source-resource {ip1_id} --target-resource {ip2_id}', checks=[
                self.check('length(metricsAlert)', 2),
            ])


class MonitorCloneStorageAccountAcrossSubsScenarios(ScenarioTest):
    @live_only()
    @ResourceGroupPreparer(name_prefix='cli_test_metric_alert_clone')
    def test_monitor_clone_storage_metric_alerts_across_subs_scenario(self, resource_group):
        self.kwargs.update({
            'alert': 'alert1',
            'sa': self.create_random_name('sa', 24),
            'sa2': self.create_random_name('sa', 24),
            'ag1': 'ag1',
            'rg': resource_group,
            'ext_sub': '1c638cf4-608f-4ee6-b680-c329e824c3a8',
            'ext_rg': self.create_random_name('test_rg', 24),
        })

        self.cmd('group create -l eastus -n {ext_rg} --subscription {ext_sub}')
        sa1_json = self.cmd('storage account create -n {sa} -g {ext_rg} -l eastus --sku Standard_LRS --kind Storage --subscription {ext_sub}').get_output_in_json()
        sa2_json = self.cmd('storage account create -n {sa2} -g {rg} -l eastus --sku Standard_LRS --kind Storage').get_output_in_json()
        self.kwargs.update({
            'sa_id': sa1_json['id'],
            'sa_id_2': sa2_json['id'],
        })
        self.cmd('monitor action-group create -g {ext_rg} -n {ag1} --subscription {ext_sub}')
        self.cmd('monitor metrics alert create -g {ext_rg} --subscription {ext_sub} -n {alert} '
                 '--scopes {sa_id} --action {ag1} --description "Test" '
                 '--condition "total transactions > 5 where ResponseType includes Success and ApiName includes GetBlob" '
                 '--condition "avg SuccessE2ELatency > 250 where ApiName includes GetBlob"',
                 checks=[
                     self.check('description', 'Test'),
                     self.check('severity', 2),
                     self.check('autoMitigate', None),
                     self.check('windowSize', '0:05:00'),
                     self.check('evaluationFrequency', '0:01:00'),
                     self.check('length(criteria.allOf)', 2),
                     self.check('length(criteria.allOf[0].dimensions)', 2),
                     self.check('length(criteria.allOf[1].dimensions)', 1)
                 ])

        with mock.patch('azure.cli.command_modules.monitor.operations.monitor_clone_util.gen_guid', side_effect=self.create_guid):
            self.cmd('monitor clone --source-resource {sa_id} --target-resource {sa_id_2}', checks=[
                self.check('metricsAlert[0].description', 'Test'),
                self.check('metricsAlert[0].severity', 2),
                self.check('metricsAlert[0].autoMitigate', None),
                self.check('metricsAlert[0].windowSize', '0:05:00'),
                self.check('metricsAlert[0].evaluationFrequency', '0:01:00'),
                self.check('length(metricsAlert[0].criteria.allOf)', 2),
                self.check('length(metricsAlert[0].criteria.allOf[0].dimensions)', 2),
                self.check('length(metricsAlert[0].criteria.allOf[1].dimensions)', 1),
            ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import time
import unittest
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer


class AzureSignalRServiceUpstreamScenarioTest(ScenarioTest):
    @ResourceGroupPreparer(random_name_length=20)
    def test_signalr_upstream(self, resource_group):
        signalr_name = self.create_random_name('signalr', 16)
        sku = 'Standard_S1'
        unit_count = 1
        location = 'eastus'
        tags_key = 'key'
        tags_val = 'value'
        service_mode = 'Classic'
        allowed_origins = ['http://example1.com', 'http://example2.com']
        updated_sku = 'Free_F1'
        updated_tags_val = 'value2'
        update_service_mode = 'Serverless'
        added_allowed_origins = ['http://example3.com', 'http://example4.com']

        self.kwargs.update({
            'location': location,
            'signalr_name': signalr_name,
            'sku': sku,
            'unit_count': unit_count,
            'tags': '{}={}'.format(tags_key, tags_val),
            'service_mode': service_mode,
            'allowed_origins': ' '.join(allowed_origins),
            'updated_sku': updated_sku,
            'updated_tags': '{}={}'.format(tags_key, updated_tags_val),
            'update_service_mode': update_service_mode,
            'added_allowed_origins': ' '.join(added_allowed_origins),
            'default_action': "Deny",
            'url_template_1': 'http://host1.com',
            'url_template_2': 'http://host2.com',
            'hub_pattern_1': 'chat',
            'hub_pattern_2': '*',
            'category_pattern_1': 'connections',
            'category_pattern_2': 'messages',
            'event_pattern_1': 'connected',
            'event_pattern_2': '*',
        })

        # Test create
        self.cmd('az signalr create -n {signalr_name} -g {rg} --sku {sku} --unit-count {unit_count} -l {location} --tags {tags} --service-mode {service_mode} --allowed-origins {allowed_origins} --default-action {default_action}',
                 checks=[
                     self.check('name', '{signalr_name}'),
                     self.check('location', '{location}'),
                     self.check('provisioningState', 'Succeeded'),
                     self.check('sku.name', '{sku}'),
                     self.check('sku.capacity', '{unit_count}'),
                     self.check('tags.{}'.format(tags_key), tags_val),
                     self.check('features[0].value', '{service_mode}'),
                     self.check('cors.allowedOrigins', allowed_origins),
                     self.exists('hostName'),
                     self.exists('publicPort'),
                     self.exists('serverPort'),
                     self.check('networkAcLs.defaultAction', '{default_action}')
                 ])

        # Test upstream update
        self.cmd('az signalr upstream update -n {signalr_name} -g {rg} --template url-template="{url_template_1}" category-pattern="{category_pattern_1}" hub-pattern={hub_pattern_1} event-pattern={event_pattern_1} --template url-template={url_template_2} category-pattern={category_pattern_2}', checks=[
            self.check('upstream.templates[0].urlTemplate', '{url_template_1}'),
            self.check('upstream.templates[0].hubPattern', '{hub_pattern_1}'),
            self.check('upstream.templates[0].categoryPattern', '{category_pattern_1}'),
            self.check('upstream.templates[0].eventPattern', '{event_pattern_1}'),
            self.check('upstream.templates[1].urlTemplate', '{url_template_2}'),
            self.check('upstream.templates[1].hubPattern', '{hub_pattern_2}'),
            self.check('upstream.templates[1].categoryPattern', '{category_pattern_2}'),
            self.check('upstream.templates[1].eventPattern', '{event_pattern_2}')
        ])

        # Test upstream list
        self.cmd('az signalr upstream list -n {signalr_name} -g {rg}', checks=[
            self.check('templates[0].urlTemplate', '{url_template_1}'),
            self.check('templates[0].hubPattern', '{hub_pattern_1}'),
            self.check('templates[0].categoryPattern', '{category_pattern_1}'),
            self.check('templates[0].eventPattern', '{event_pattern_1}'),
            self.check('templates[1].urlTemplate', '{url_template_2}'),
            self.check('templates[1].hubPattern', '{hub_pattern_2}'),
            self.check('templates[1].categoryPattern', '{category_pattern_2}'),
            self.check('templates[1].eventPattern', '{event_pattern_2}')
        ])

        # Test upstream clear
        self.cmd('az signalr upstream clear -n {signalr_name} -g {rg}', checks=[
            self.check('length(upstream.templates)', 0)
        ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
from azure.cli.testsdk import (LiveScenarioTest, ResourceGroupPreparer, StorageAccountPreparer,
                               JMESPathCheck, live_only)
from ..storage_test_util import StorageScenarioMixin


class StorageSASScenario(StorageScenarioMixin, LiveScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_storage_file_sas_scenario(self, resource_group, storage_account):
        from datetime import datetime, timedelta
        expiry = (datetime.utcnow() + timedelta(hours=1)).strftime('%Y-%m-%dT%H:%MZ')

        account_info = self.get_account_info(resource_group, storage_account)
        share = self.create_share(account_info)
        local_file = self.create_temp_file(128, full_random=False)
        file_name = self.create_random_name('file', 16) + '.txt'

        self.kwargs.update({
            'expiry': expiry,
            'account': storage_account,
            'share': share,
            'local_file': local_file,
            'file': file_name
        })

        # test sas-token for a fileshare with account key
        sas = self.storage_cmd('storage share generate-sas -n {} --https-only --permissions dlrw --expiry {} -otsv',
                               account_info, share, expiry).output.strip()
        self.kwargs['share_sas'] = sas
        self.cmd('storage file upload --share-name {share} --source "{local_file}" -p {file} '
                 '--account-name {account} --sas-token "{share_sas}"')

        # test sas-token for a file with account key
        sas = self.storage_cmd(
            'storage file generate-sas -s {} -p {} --https-only --permissions cdrw --expiry {} -otsv',
            account_info, share, file_name, expiry).output.strip()
        self.kwargs['file_sas'] = sas
        self.cmd('storage file show -s {share} -p {file} --account-name {account} --sas-token {file_sas}') \
            .assert_with_checks(JMESPathCheck('name', file_name))

        # connection string
        connection_str = self.cmd('storage account show-connection-string -n {account}  --query connectionString '
                                  '-otsv').output.strip()
        self.kwargs['con_str'] = connection_str
        # test sas-token for a fileshare with connection string
        sas = self.cmd('storage share generate-sas -n {share} --https-only --permissions dlrw --expiry {expiry} '
                       '--connection-string "{con_str}" -otsv').output.strip()
        self.kwargs['share_sas'] = sas
        self.cmd('storage file upload --share-name {share} --source "{local_file}" -p {file} '
                 '--account-name {account} --sas-token "{share_sas}"')

        # test sas-token for a file with connection string
        sas = self.cmd(
            'storage file generate-sas -s {share} -p {file} --https-only --permissions cdrw --expiry {expiry} '
            '--connection-string {con_str} -otsv').output.strip()
        self.kwargs['file_sas'] = sas
        self.cmd('storage file show -s {share} -p {file} --account-name {account} --sas-token {file_sas}') \
            .assert_with_checks(JMESPathCheck('name', file_name))

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_storage_blob_sas_scenario(self, resource_group, storage_account):
        from datetime import datetime, timedelta
        expiry = (datetime.utcnow() + timedelta(hours=1)).strftime('%Y-%m-%dT%H:%MZ')

        account_info = self.get_account_info(resource_group, storage_account)
        container = self.create_container(account_info)
        local_file = self.create_temp_file(128, full_random=False)
        blob_name = self.create_random_name('blob', 16)

        self.kwargs.update({
            'expiry': expiry,
            'account': storage_account,
            'container': container,
            'local_file': local_file,
            'blob': blob_name
        })

        # account key
        # test sas-token for a container
        sas = self.storage_cmd('storage container generate-sas -n {} --https-only --permissions dlrw --expiry {} -otsv',
                               account_info, container, expiry).output.strip()
        self.kwargs['container_sas'] = sas
        self.cmd('storage blob upload -c {container} -f "{local_file}" -n {blob} '
                 '--account-name {account} --sas-token "{container_sas}"')

        # test sas-token for a file
        sas = self.storage_cmd('storage blob generate-sas -c {} -n {} --https-only --permissions acdrw --expiry {} '
                               '-otsv', account_info, container, blob_name, expiry).output.strip()
        self.kwargs['blob_sas'] = sas
        self.cmd('storage blob show -c {container} -n {blob} --account-name {account} --sas-token {blob_sas}') \
            .assert_with_checks(JMESPathCheck('name', blob_name))

        # connection string
        connection_str = self.cmd('storage account show-connection-string -n {account}  --query connectionString '
                                  '-otsv').output.strip()
        self.kwargs['con_str'] = connection_str
        # test sas-token for a container
        sas = self.cmd('storage container generate-sas -n {container} --https-only --permissions dlrw '
                       '--connection-string {con_str} --expiry {expiry} -otsv').output.strip()
        self.kwargs['container_sas'] = sas
        self.cmd('storage blob upload -c {container} -f "{local_file}" -n {blob} '
                 '--account-name {account} --sas-token "{container_sas}"')

        # test sas-token for a blob
        sas = self.cmd('storage blob generate-sas -c {container} -n {blob} --account-name {account} --https-only '
                       '--permissions acdrw --expiry {expiry} -otsv').output.strip()
        self.kwargs['blob_sas'] = sas
        self.cmd('storage blob show -c {container} -n {blob} --account-name {account} --sas-token {blob_sas}') \
            .assert_with_checks(JMESPathCheck('name', blob_name))

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_storage_queue_sas_scenario(self, resource_group, storage_account):
        from datetime import datetime, timedelta
        expiry = (datetime.utcnow() + timedelta(hours=1)).strftime('%Y-%m-%dT%H:%MZ')
        account_info = self.get_account_info(resource_group, storage_account)
        queue = self.create_random_name('queue', 24)

        self.storage_cmd('storage queue create -n {} --fail-on-exist --metadata a=b c=d', account_info, queue)\
            .assert_with_checks(JMESPathCheck('created', True))

        sas = self.storage_cmd('storage queue generate-sas -n {} --permissions r --expiry {}',
                               account_info, queue, expiry).output
        self.assertIn('sig', sas, 'The sig segment is not in the sas {}'.format(sas))

        self.cmd('storage queue exists -n {} --account-name {} --sas-token {}'.format(queue, storage_account, sas),
                 checks=JMESPathCheck('exists', True))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
import os
from datetime import datetime, timedelta

from azure.cli.testsdk import CliTestError, ResourceGroupPreparer
from azure.cli.testsdk.preparers import AbstractPreparer, SingleValueReplacer
from azure.cli.testsdk.base import execute
# pylint: disable=line-too-long


class VaultPreparer(AbstractPreparer, SingleValueReplacer):  # pylint: disable=too-many-instance-attributes
    def __init__(self, name_prefix='clitest-vault', parameter_name='vault_name',
                 resource_group_location_parameter_name='resource_group_location',
                 resource_group_parameter_name='resource_group',
                 dev_setting_name='AZURE_CLI_TEST_DEV_BACKUP_ACCT_NAME', soft_delete=True):
        super(VaultPreparer, self).__init__(name_prefix, 24)
        from azure.cli.core.mock import DummyCli
        self.cli_ctx = DummyCli()
        self.parameter_name = parameter_name
        self.resource_group = None
        self.resource_group_parameter_name = resource_group_parameter_name
        self.location = None
        self.resource_group_location_parameter_name = resource_group_location_parameter_name
        self.dev_setting_value = os.environ.get(dev_setting_name, None)
        self.soft_delete = soft_delete

    def create_resource(self, name, **kwargs):
        if not self.dev_setting_value:
            self.resource_group = self._get_resource_group(**kwargs)
            self.location = self._get_resource_group_location(**kwargs)
            cmd = 'az backup vault create -n {} -g {} --location {}'.format(name, self.resource_group, self.location)
            execute(self.cli_ctx, cmd)
            if not self.soft_delete:
                cmd = 'az backup vault backup-properties set -n {} -g {} --soft-delete-feature-state Disable'.format(name, self.resource_group)
                execute(self.cli_ctx, cmd)
            return {self.parameter_name: name}
        return {self.parameter_name: self.dev_setting_value}

    def remove_resource(self, name, **kwargs):
        self._cleanup(name, self.resource_group)

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create a vault, a resource group is required. Please add ' \
                       'decorator @{} in front of this Vault preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))

    def _get_resource_group_location(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_location_parameter_name)
        except KeyError:
            template = 'To create a vault, a resource group is required. Please add ' \
                       'decorator @{} in front of this Vault preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))

    def _cleanup(self, vault_name, resource_group):
        containers = execute(self.cli_ctx, 'az backup container list --backup-management-type AzureIaasVM -v {} -g {} --query [].properties.friendlyName'
                             .format(vault_name, resource_group)).get_output_in_json()
        for container in containers:
            items = execute(self.cli_ctx, 'az backup item list --backup-management-type AzureIaasVM --workload-type VM -g {} -v {} -c {} --query [].properties.friendlyName'
                            .format(resource_group, vault_name, container)).get_output_in_json()
            for item in items:
                execute(self.cli_ctx,
                        'az backup protection disable --backup-management-type AzureIaasVM --workload-type VM -g {} -v {} -c {} -i {} --delete-backup-data true --yes'
                        .format(resource_group, vault_name, container, item))
        from msrestazure.azure_exceptions import CloudError
        try:
            execute(self.cli_ctx, 'az backup vault delete -n {} -g {} --yes'.format(vault_name, resource_group))
        except CloudError as ex:
            if 'Recovery Services vault cannot be deleted as there are backup items in soft deleted state in the vault' not in str(ex):
                raise ex


class VMPreparer(AbstractPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest-vm', parameter_name='vm_name',
                 resource_group_location_parameter_name='resource_group_location',
                 resource_group_parameter_name='resource_group', dev_setting_name='AZURE_CLI_TEST_DEV_BACKUP_VM_NAME'):
        super(VMPreparer, self).__init__(name_prefix, 15)
        from azure.cli.core.mock import DummyCli
        self.cli_ctx = DummyCli()
        self.parameter_name = parameter_name
        self.resource_group = None
        self.resource_group_parameter_name = resource_group_parameter_name
        self.location = None
        self.resource_group_location_parameter_name = resource_group_location_parameter_name
        self.dev_setting_value = os.environ.get(dev_setting_name, None)

    def create_resource(self, name, **kwargs):
        if not self.dev_setting_value:
            self.resource_group = self._get_resource_group(**kwargs)
            self.location = self._get_resource_group_location(**kwargs)
            param_format = '-n {} -g {} --image {} --admin-username {} --admin-password {} --tags {} --nsg-rule None'
            param_tags = 'MabUsed=Yes Owner=sisi Purpose=CLITest DeleteBy=12-2099 AutoShutdown=No'
            param_string = param_format.format(name, self.resource_group, 'Win2012R2Datacenter', name,
                                               '%j^VYw9Q3Z@Cu$*h', param_tags)
            cmd = 'az vm create {}'.format(param_string)
            execute(self.cli_ctx, cmd)
            return {self.parameter_name: name}
        return {self.parameter_name: self.dev_setting_value}

    def remove_resource(self, name, **kwargs):
        # Resource group deletion will take care of this.
        pass

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create a vm, a resource group is required. Please add ' \
                       'decorator @{} in front of this VM preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))

    def _get_resource_group_location(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_location_parameter_name)
        except KeyError:
            template = 'To create a vm, a resource group is required. Please add ' \
                       'decorator @{} in front of this VM preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))


class ItemPreparer(AbstractPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest-item', parameter_name='item_name', vm_parameter_name='vm_name',
                 vault_parameter_name='vault_name',
                 resource_group_parameter_name='resource_group',
                 dev_setting_name='AZURE_CLI_TEST_DEV_BACKUP_ITEM_NAME'):
        super(ItemPreparer, self).__init__(name_prefix, 24)
        from azure.cli.core.mock import DummyCli
        self.cli_ctx = DummyCli()
        self.parameter_name = parameter_name
        self.vm_parameter_name = vm_parameter_name
        self.resource_group = None
        self.resource_group_parameter_name = resource_group_parameter_name
        self.vault_parameter_name = vault_parameter_name
        self.dev_setting_value = os.environ.get(dev_setting_name, None)

    def create_resource(self, name, **kwargs):
        if not self.dev_setting_value:
            self.resource_group = self._get_resource_group(**kwargs)
            vault = self._get_vault(**kwargs)
            vm = self._get_vm(**kwargs)

            params_format = '-g {} -v {} --vm {} -p DefaultPolicy'
            param_string = params_format.format(self.resource_group, vault, vm)
            execute(self.cli_ctx, 'az backup protection enable-for-vm {}'.format(param_string))
            return {self.parameter_name: name}
        return {self.parameter_name: self.dev_setting_value}

    def remove_resource(self, name, **kwargs):
        # Vault deletion will take care of this.
        pass

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create an item, a resource group is required. Please add ' \
                       'decorator @{} in front of this Item preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))

    def _get_vault(self, **kwargs):
        try:
            return kwargs.get(self.vault_parameter_name)
        except KeyError:
            template = 'To create an item, a vault is required. Please add ' \
                       'decorator @{} in front of this Item preparer.'
            raise CliTestError(template.format(VaultPreparer.__name__,
                                               self.vault_parameter_name))

    def _get_vm(self, **kwargs):
        try:
            return kwargs.get(self.vm_parameter_name)
        except KeyError:
            template = 'To create an item, a vm is required. Please add ' \
                       'decorator @{} in front of this Item preparer.'
            raise CliTestError(template.format(VMPreparer.__name__,
                                               self.vm_parameter_name))


class PolicyPreparer(AbstractPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest-item', parameter_name='policy_name', vault_parameter_name='vault_name',
                 resource_group_parameter_name='resource_group',
                 instant_rp_days=None):
        super(PolicyPreparer, self).__init__(name_prefix, 24)
        from azure.cli.core.mock import DummyCli
        self.cli_ctx = DummyCli()
        self.parameter_name = parameter_name
        self.resource_group = None
        self.resource_group_parameter_name = resource_group_parameter_name
        self.vault = None
        self.vault_parameter_name = vault_parameter_name
        self.instant_rp_days = instant_rp_days

    def create_resource(self, name, **kwargs):
        if not os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_POLICY_NAME', None):
            self.resource_group = self._get_resource_group(**kwargs)
            self.vault = self._get_vault(**kwargs)

            policy_json = execute(self.cli_ctx, 'az backup policy show -g {} -v {} -n {}'
                                  .format(self.resource_group, self.vault, 'DefaultPolicy')).get_output_in_json()
            policy_json['name'] = name
            if self.instant_rp_days:
                policy_json['properties']['instantRpRetentionRangeInDays'] = self.instant_rp_days
            policy_json = json.dumps(policy_json)

            execute(self.cli_ctx, 'az backup policy set -g {} -v {} --policy \'{}\''.format(self.resource_group,
                                                                                            self.vault,
                                                                                            policy_json))
            return {self.parameter_name: name}
        return {self.parameter_name: os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_POLICY_NAME', None)}

    def remove_resource(self, name, **kwargs):
        # Vault deletion will take care of this.
        pass

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create an item, a resource group is required. Please add ' \
                       'decorator @{} in front of this Policy preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))

    def _get_vault(self, **kwargs):
        try:
            return kwargs.get(self.vault_parameter_name)
        except KeyError:
            template = 'To create an item, a vault is required. Please add ' \
                       'decorator @{} in front of this Policy preparer.'
            raise CliTestError(template.format(VaultPreparer.__name__,
                                               self.vault_parameter_name))


class RPPreparer(AbstractPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest-rp', parameter_name='rp_name', vm_parameter_name='vm_name',
                 vault_parameter_name='vault_name',
                 resource_group_parameter_name='resource_group', dev_setting_name='AZURE_CLI_TEST_DEV_BACKUP_RP_NAME'):
        super(RPPreparer, self).__init__(name_prefix, 24)
        from azure.cli.core.mock import DummyCli
        self.cli_ctx = DummyCli()
        self.parameter_name = parameter_name
        self.vm_parameter_name = vm_parameter_name
        self.resource_group = None
        self.resource_group_parameter_name = resource_group_parameter_name
        self.vault_parameter_name = vault_parameter_name
        self.dev_setting_value = os.environ.get(dev_setting_name, None)

    def create_resource(self, name, **kwargs):
        if not self.dev_setting_value:
            self.resource_group = self._get_resource_group(**kwargs)
            vault = self._get_vault(**kwargs)
            vm = self._get_vm(**kwargs)

            retain_date = datetime.utcnow() + timedelta(days=30)
            command_string = 'az backup protection backup-now --backup-management-type AzureIaasVM --workload-type VM -g {} -v {} -c {} -i {} --retain-until {} --query name'
            command_string = command_string.format(self.resource_group, vault, vm, vm,
                                                   retain_date.strftime('%d-%m-%Y'))
            backup_job = execute(self.cli_ctx, command_string).get_output_in_json()
            execute(self.cli_ctx, 'az backup job wait -g {} -v {} -n {}'.format(self.resource_group, vault, backup_job))
            return {self.parameter_name: name}
        return {self.parameter_name: self.dev_setting_value}

    def remove_resource(self, name, **kwargs):
        # Vault deletion will take care of this.
        pass

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create an item, a resource group is required. Please add ' \
                       'decorator @{} in front of this RP preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))

    def _get_vault(self, **kwargs):
        try:
            return kwargs.get(self.vault_parameter_name)
        except KeyError:
            template = 'To create an item, a vault is required. Please add ' \
                       'decorator @{} in front of this RP preparer.'
            raise CliTestError(template.format(VaultPreparer.__name__,
                                               self.vault_parameter_name))

    def _get_vm(self, **kwargs):
        try:
            return kwargs.get(self.vm_parameter_name)
        except KeyError:
            template = 'To create an rp, a VM is required. Please add ' \
                       'decorator @{} in front of this RP preparer.'
            raise CliTestError(template.format(ItemPreparer.__name__, self.vm_parameter_name))


class AFSPolicyPreparer(AbstractPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest-item', parameter_name='policy_name', vault_parameter_name='vault_name',
                 resource_group_parameter_name='resource_group',
                 instant_rp_days=None):
        super(AFSPolicyPreparer, self).__init__(name_prefix, 24)
        from azure.cli.core.mock import DummyCli
        self.cli_ctx = DummyCli()
        self.parameter_name = parameter_name
        self.resource_group = None
        self.resource_group_parameter_name = resource_group_parameter_name
        self.vault = None
        self.vault_parameter_name = vault_parameter_name
        self.instant_rp_days = instant_rp_days

    def create_resource(self, name, **kwargs):
        if not os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_POLICY_NAME', None):
            self.resource_group = self._get_resource_group(**kwargs)
            self.vault = self._get_vault(**kwargs)

            policy_json = execute(self.cli_ctx, 'az backup policy show -g {} -v {} -n {}'
                                  .format(self.resource_group, self.vault, 'DefaultPolicy')).get_output_in_json()
            policy_json['name'] = name
            if self.instant_rp_days:
                policy_json['properties']['instantRpRetentionRangeInDays'] = self.instant_rp_days
            policy_json['properties']['backupManagementType'] = "AzureStorage"
            policy_json = json.dumps(policy_json)

            command_string = 'az backup policy create -g {} -v {} --policy \'{}\' -n {} --backup-management-type {}'
            command_string = command_string.format(self.resource_group, self.vault, policy_json, name, "AzureStorage")
            execute(self.cli_ctx, command_string)
            return {self.parameter_name: name}
        return {self.parameter_name: os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_POLICY_NAME', None)}

    def remove_resource(self, name, **kwargs):
        # Vault deletion will take care of this.
        pass

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create an item, a resource group is required. Please add ' \
                       'decorator @{} in front of this Policy preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))

    def _get_vault(self, **kwargs):
        try:
            return kwargs.get(self.vault_parameter_name)
        except KeyError:
            template = 'To create an item, a vault is required. Please add ' \
                       'decorator @{} in front of this Policy preparer.'
            raise CliTestError(template.format(VaultPreparer.__name__,
                                               self.vault_parameter_name))


class FileSharePreparer(AbstractPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest-item', storage_account_parameter_name='storage_account',
                 resource_group_parameter_name='resource_group', file_parameter_name=None,
                 parameter_name='afs_name', file_upload=False):
        super(FileSharePreparer, self).__init__(name_prefix, 24)
        from azure.cli.core.mock import DummyCli
        self.cli_ctx = DummyCli()
        self.parameter_name = parameter_name
        self.resource_group = None
        self.resource_group_parameter_name = resource_group_parameter_name
        self.storage_account_parameter_name = storage_account_parameter_name
        self.file_parameter_name = file_parameter_name
        self.file_upload = file_upload

    def create_resource(self, name, **kwargs):
        if not os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_POLICY_NAME', None):
            self.resource_group = self._get_resource_group(**kwargs)
            storage_account = self._get_storage_account(**kwargs)

            storage_keys_command = 'az storage account keys list --resource-group {} --account-name {} --query [0].value'
            storage_keys_command = storage_keys_command.format(self.resource_group, storage_account)
            if self.file_upload:
                storage_key = execute(self.cli_ctx, storage_keys_command).get_output_in_json()
            connection_string_command = 'az storage account show-connection-string -n {} -g {}'
            connection_string_command = connection_string_command.format(storage_account, self.resource_group)
            connection_string = execute(self.cli_ctx, connection_string_command).get_output_in_json()
            connection_string = connection_string['connectionString']
            command_string = 'az storage share create --name {} --quota 1 --connection-string {}'
            command_string = command_string.format(name, connection_string)
            execute(self.cli_ctx, command_string)
            file_upload_command_format = 'az storage file upload --account-name {} --account-key {} --share-name {} --source {}'
            if self.file_upload:
                file_param_names = self.file_parameter_name
                for file_param_name in file_param_names:
                    self.file_parameter_name = file_param_name
                    file = self._get_file(**kwargs)
                    file_upload_command = file_upload_command_format.format(storage_account, storage_key, name, file)
                    execute(self.cli_ctx, file_upload_command)
            return {self.parameter_name: name}
        return {self.parameter_name: os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_POLICY_NAME', None)}

    def remove_resource(self, name, **kwargs):
        # Vault deletion will take care of this.
        pass

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create an item, a resource group is required. Please add ' \
                       'decorator @{} in front of this Policy preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))

    def _get_file(self, **kwargs):
        try:
            return kwargs.get(self.file_parameter_name)
        except KeyError:
            raise CliTestError("File not Found")

    def _get_storage_account(self, **kwargs):
        try:
            return kwargs.get(self.storage_account_parameter_name)
        except KeyError:
            template = 'To create an item, a vault is required. Please add ' \
                       'decorator @StorageAccountPreparer in front of this Policy preparer.'
            raise CliTestError(template)


class AFSItemPreparer(AbstractPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest-item', storage_account_parameter_name='storage_account',
                 resource_group_parameter_name='resource_group', vault_parameter_name='vault_name',
                 parameter_name='item_name', afs_parameter_name='afs_name',
                 policy_parameter_name='policy_name'):
        super(AFSItemPreparer, self).__init__(name_prefix, 24)
        from azure.cli.core.mock import DummyCli
        self.cli_ctx = DummyCli()
        self.parameter_name = parameter_name
        self.resource_group_parameter_name = resource_group_parameter_name
        self.storage_account_parameter_name = storage_account_parameter_name
        self.vault_parameter_name = vault_parameter_name
        self.afs_parameter_name = afs_parameter_name
        self.policy_parameter_name = policy_parameter_name

    def create_resource(self, name, **kwargs):
        if not os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_ITEM_NAME', None):
            resource_group = self._get_resource_group(**kwargs)
            storage_account = self._get_storage_account(**kwargs)
            vault = self._get_vault(**kwargs)
            afs = self._get_file_share(**kwargs)
            policy = self._get_policy(**kwargs)

            command_string = 'az backup protection enable-for-azurefileshare'
            command_string += ' -g {} -v {} --azure-file-share {} --storage-account {} -p {}'
            command_string = command_string.format(resource_group, vault, afs, storage_account, policy)
            execute(self.cli_ctx, command_string)
            return {self.parameter_name: name}
        return {self.parameter_name: os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_ITEM_NAME', None)}

    def remove_resource(self, name, **kwargs):
        # Vault deletion will take care of this.
        pass

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create an item, a resource group is required. Please add ' \
                       'decorator @{} in front of this Policy preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))

    def _get_vault(self, **kwargs):
        try:
            return kwargs.get(self.vault_parameter_name)
        except KeyError:
            template = 'To create an item, a vault is required. Please add ' \
                       'decorator @{} in front of this Policy preparer.'
            raise CliTestError(template.format(VaultPreparer.__name__,
                                               self.vault_parameter_name))

    def _get_storage_account(self, **kwargs):
        try:
            return kwargs.get(self.storage_account_parameter_name)
        except KeyError:
            template = 'To create an item, a vault is required. Please add ' \
                       'decorator @StorageAccountPreparer in front of this Policy preparer.'
            raise CliTestError(template)

    def _get_file_share(self, **kwargs):
        try:
            return kwargs.get(self.afs_parameter_name)
        except KeyError:
            template = 'To create an item, a fileshare is required. Please add ' \
                       'decorator @FileSharePreparer in front of this Policy preparer.'
            raise CliTestError(template)

    def _get_policy(self, **kwargs):
        try:
            return kwargs.get(self.policy_parameter_name)
        except KeyError:
            template = 'To create an item, a policy is required. Please add ' \
                       'decorator @AFSPolicyPreparer in front of this Policy preparer.'
            raise CliTestError(template)


class AFSRPPreparer(AbstractPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest-item', storage_account_parameter_name='storage_account',
                 resource_group_parameter_name='resource_group', vault_parameter_name='vault_name',
                 parameter_name='rp_name', afs_parameter_name='afs_name'):
        super(AFSRPPreparer, self).__init__(name_prefix, 24)
        from azure.cli.core.mock import DummyCli
        self.cli_ctx = DummyCli()
        self.parameter_name = parameter_name
        self.resource_group = None
        self.resource_group_parameter_name = resource_group_parameter_name
        self.storage_account_parameter_name = storage_account_parameter_name
        self.vault_parameter_name = vault_parameter_name
        self.afs_parameter_name = afs_parameter_name

    def create_resource(self, name, **kwargs):
        if not os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_RP_NAME', None):
            self.resource_group = self._get_resource_group(**kwargs)
            storage_account = self._get_storage_account(**kwargs)
            vault = self._get_vault(**kwargs)
            afs = self._get_file_share(**kwargs)

            retain_date = datetime.utcnow() + timedelta(days=30)
            retain_date = retain_date.strftime('%d-%m-%Y')
            command_string = 'az backup protection backup-now'
            command_string += ' -g {} -v {} -i {} -c {} --backup-management-type AzureStorage --retain-until {} --query name'
            command_string = command_string.format(self.resource_group, vault, afs, storage_account, retain_date)
            backup_job = execute(self.cli_ctx, command_string).get_output_in_json()
            execute(self.cli_ctx, 'az backup job wait -g {} -v {} -n {}'.format(self.resource_group, vault, backup_job))
            return {self.parameter_name: name}
        return {self.parameter_name: os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_RP_NAME', None)}

    def remove_resource(self, name, **kwargs):
        # Vault deletion will take care of this.
        pass

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create an item, a resource group is required. Please add ' \
                       'decorator @{} in front of this Policy preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))

    def _get_vault(self, **kwargs):
        try:
            return kwargs.get(self.vault_parameter_name)
        except KeyError:
            template = 'To create an item, a vault is required. Please add ' \
                       'decorator @{} in front of this Policy preparer.'
            raise CliTestError(template.format(VaultPreparer.__name__,
                                               self.vault_parameter_name))

    def _get_storage_account(self, **kwargs):
        try:
            return kwargs.get(self.storage_account_parameter_name)
        except KeyError:
            template = 'To create a RP, an item is required. Please add ' \
                       'decorator @AFSItemPreparer in front of this Policy preparer.'
            raise CliTestError(template)

    def _get_file_share(self, **kwargs):
        try:
            return kwargs.get(self.afs_parameter_name)
        except KeyError:
            template = 'To create an item, a fileshare is required. Please add ' \
                       'decorator @FileSharePreparer in front of this Policy preparer.'
            raise CliTestError(template)


class FilePreparer(AbstractPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clitest-file', parameter_name='file_name'):
        super(FilePreparer, self).__init__(name_prefix, 24)
        self.parameter_name = parameter_name

    def create_resource(self, name, **kwargs):
        if not os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_RP_NAME', None):
            f = open(name, "a")
            f.close()
            return {self.parameter_name: name}
        return {self.parameter_name: os.environ.get('AZURE_CLI_TEST_DEV_BACKUP_RP_NAME', None)}

    def remove_resource(self, name, **kwargs):
        os.remove(name)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import sys
import os
import subprocess
import automation.utilities.path as autmation_path

INSTALL_COMMAND = 'python -m pip install -e {}'


def install_modules():
    all_modules = list(autmation_path.get_command_modules_paths())

    print('Installing command modules')
    print('Modules: {}'.format(', '.join(name for name, _ in all_modules)))
    failures = []

    for name, path in all_modules:
        try:
            subprocess.check_call(INSTALL_COMMAND.format(path).split())
        except subprocess.CalledProcessError as err:
            # exit code is not zero
            failures.append('Failed to install {}. Error message: {}'.format(name, err.output))

    for f in failures:
        print(f)

    return not any(failures)


if __name__ == '__main__':
    sys.exit(0 if install_modules() else 1)



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: disable=unused-argument

from .. import try_manual


# EXAMPLE: /Devices/put/DataBoxEdgeDevicePut
@try_manual
def step_device_create(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge device create '
             '--location "eastus" '
             '--sku "Edge" '
             '--name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Devices/get/DataBoxEdgeDeviceGetByName
@try_manual
def step_device_show(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge device show '
             '--name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Devices/get/DataBoxEdgeDeviceGetByResourceGroup
@try_manual
def step_device_list(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge device list '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Devices/get/DataBoxEdgeDeviceGetBySubscription
@try_manual
def step_device_list2(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge device list '
             '-g ""',
             checks=checks)


# EXAMPLE: /Devices/get/UpdateSummaryGet
@try_manual
def step_device_show_update_summary(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge device show-update-summary '
             '--name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Devices/patch/DataBoxEdgeDevicePatch
@try_manual
def step_device_update(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge device update '
             '--name "{myDevice}" '
             '--tags Key1="value1" Key2="value2" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Devices/post/DownloadUpdatesPost
@try_manual
def step_device_download_update(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge device download-update '
             '--name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Devices/post/InstallUpdatesPost
@try_manual
def step_device_install_update(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge device install-update '
             '--name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Devices/post/ScanForUpdatesPost
@try_manual
def step_device_scan_for_update(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge device scan-for-update '
             '--name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Alerts/get/AlertGet
@try_manual
def step_alert_show(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge alert show '
             '--name "159a00c7-8543-4343-9435-263ac87df3bb" '
             '--device-name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Alerts/get/AlertGetAllInDevice
@try_manual
def step_alert_list(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge alert list '
             '--device-name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /BandwidthSchedules/put/BandwidthSchedulePut
@try_manual
def step_bandwidth_schedule_create(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge bandwidth-schedule create '
             '--name "bandwidth-1" '
             '--device-name "{myDevice}" '
             '--days "Sunday" "Monday" '
             '--rate-in-mbps 100 '
             '--start "0:0:0" '
             '--stop "13:59:0" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /BandwidthSchedules/get/BandwidthScheduleGet
@try_manual
def step_bandwidth_schedule_show(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge bandwidth-schedule show '
             '--name "bandwidth-1" '
             '--device-name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /BandwidthSchedules/get/BandwidthScheduleGetAllInDevice
@try_manual
def step_bandwidth_schedule_list(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge bandwidth-schedule list '
             '--device-name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /BandwidthSchedules/delete/BandwidthScheduleDelete
@try_manual
def step_bandwidth_schedule_delete(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge bandwidth-schedule delete -y '
             '--name "bandwidth-1" '
             '--device-name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Jobs/get/JobsGet
@try_manual
def step_show_job(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge show-job '
             '--name "159a00c7-8543-4343-9435-263ac87df3bb" '
             '--device-name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Nodes/get/NodesGetAllInDevice
@try_manual
def step_list_node(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge list-node '
             '--device-name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Orders/put/OrderPut
@try_manual
def step_order_create(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge order create '
             '--device-name "{myDevice}" '
             '--company-name "Microsoft" '
             '--contact-person "John Mcclane" '
             '--email-list "john@microsoft.com" '
             '--phone "(800) 426-9400" '
             '--address-line1 "Microsoft Corporation" '
             '--address-line2 "One Microsoft Way" '
             '--address-line3 "Redmond" '
             '--city "WA" '
             '--country "United States" '
             '--postal-code "98052" '
             '--state "WA" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Orders/get/OrderGet
@try_manual
def step_order_show(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge order show '
             '--device-name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Orders/get/OrderGetAllInDevice
@try_manual
def step_order_list(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge order list '
             '--device-name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Orders/delete/OrderDelete
@try_manual
def step_order_delete(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge order delete -y '
             '--device-name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Devices/delete/DataBoxEdgeDeviceDelete
@try_manual
def step_device_delete(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge device delete -y '
             '--name "{myDevice}" '
             '--resource-group "{rg}"',
             checks=checks)


# EXAMPLE: /Skus/get/ListSkus
@try_manual
def step_list_sku(test, rg, checks=None):
    if checks is None:
        checks = []
    test.cmd('az databoxedge list-sku',
             checks=checks)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest
from azure_devtools.scenario_tests import AllowLargeResponse


class SecurityCenterRegulatoryComplianceTests(ScenarioTest):

    @AllowLargeResponse()
    def test_security_compliance(self):

        standards_list = self.cmd('az security regulatory-compliance-standards list').get_output_in_json()
        assert len(standards_list) > 0

        standard = self.cmd('az security regulatory-compliance-standards show -n "Azure-CIS-1.1.0"').get_output_in_json()
        assert len(standard) > 0

        controls_list = self.cmd('az security regulatory-compliance-standards show -n "Azure-CIS-1.1.0"').get_output_in_json()
        assert len(controls_list) > 0

        control = self.cmd('az security regulatory-compliance-controls show --standard-name "Azure-CIS-1.1.0" -n "1.1"').get_output_in_json()
        assert len(control) > 0

        assessments_list = self.cmd('az security regulatory-compliance-assessments list --standard-name "Azure-CIS-1.1.0" --control-name "1.1"').get_output_in_json()
        assert len(assessments_list) > 0

        assessment = self.cmd('az security regulatory-compliance-assessments show --standard-name "Azure-CIS-1.1.0" --control-name "1.1" -n "94290b00-4d0c-d7b4-7cea-064a9554e681"').get_output_in_json()
        assert len(assessment) > 0



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest


class TestCoreCLIOutput(unittest.TestCase):
    def test_create_AzOutputProducer(self):
        from azure.cli.core._output import AzOutputProducer
        from azure.cli.core.mock import DummyCli

        output_producer = AzOutputProducer(DummyCli())
        self.assertEqual(7, len(output_producer._FORMAT_DICT))  # six types: json, jsonc, table, tsv, yaml, yamlc, none
        self.assertIn('yaml', output_producer._FORMAT_DICT)
        self.assertIn('none', output_producer._FORMAT_DICT)

    # regression test for https://github.com/Azure/azure-cli/issues/9263
    def test_yaml_output_with_ordered_dict(self):
        from azure.cli.core._output import AzOutputProducer
        from azure.cli.core.mock import DummyCli
        from knack.util import CommandResultItem
        from collections import OrderedDict
        import yaml

        account_dict = {
            "environmentName": "AzureCloud",
            "id": "000000-000000",
            "isDefault": True,
            "name": "test_sub",
            "state": "Enabled",
            "tenantId": "000000-000000-000000",
            "user": {
                "name": "test@example.com",
                "type": "user"
            }
        }

        output_producer = AzOutputProducer(DummyCli())
        yaml_output = output_producer.get_formatter('yaml')(CommandResultItem(result=OrderedDict(account_dict)))
        self.assertEqual(account_dict, yaml.safe_load(yaml_output))


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.testsdk import ResourceGroupPreparer, JMESPathCheck
from azure.cli.testsdk import ScenarioTest
from .scenario_mixin import CdnScenarioMixin
from azure.mgmt.cdn.models import (SkuName, CustomHttpsProvisioningState, ProtocolType,
                                   CertificateType, ErrorResponseException)


class CdnCustomDomainScenarioTest(CdnScenarioMixin, ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_test_cdn_domain')
    def test_cdn_custom_domain_errors(self, resource_group):
        from azure.mgmt.cdn.models import ErrorResponseException
        from knack.util import CLIError

        self.kwargs.update({
            'profile': 'cdnprofile1',
            'endpoint': self.create_random_name(prefix='endpoint', length=24),
            'origin': 'www.test.com',
            'hostname': 'www.example.com',
            'name': 'customdomain1',
            'rg': resource_group,
        })

        self.cmd('cdn profile create -g {rg} -n {profile}')
        self.cmd('cdn endpoint create -g {rg} --origin {origin} --profile-name {profile} -n {endpoint}')
        self.cmd('cdn custom-domain list -g {rg} --endpoint-name {endpoint} --profile-name {profile}')

        # These will all fail because we don't really have the ability to create the custom endpoint in test.
        # but they should still fail if there was a CLI-level regression.
        with self.assertRaises(ErrorResponseException):
            self.cmd(
                'cdn custom-domain create -g {rg} --endpoint-name {endpoint} --hostname {hostname} --profile-name {profile} -n {name}')
        with self.assertRaises(SystemExit):  # exits with code 3 due to missing resource
            self.cmd('cdn custom-domain show -g {rg} --endpoint-name {endpoint} --profile-name {profile} -n {name}')
        self.cmd('cdn custom-domain delete -g {rg} --endpoint-name {endpoint} --profile-name {profile} -n {name}')
        with self.assertRaises(ErrorResponseException):
            self.cmd(
                'cdn custom-domain enable-https -g {rg} --endpoint-name {endpoint} --profile-name {profile} -n {name}')
        with self.assertRaises(CLIError):
            self.cmd(
                'cdn custom-domain disable-https -g {rg} --endpoint-name {endpoint} --profile-name {profile} -n {name}')

    @ResourceGroupPreparer()
    def test_cdn_custom_domain_crud(self, resource_group):
        profile_name = 'profile123'
        self.endpoint_list_cmd(resource_group, profile_name, expect_failure=True)

        self.profile_create_cmd(resource_group, profile_name, sku=SkuName.standard_akamai.value)
        # Endpoint name and custom domain hostname are hard-coded because of
        # custom domain CNAME requirement. If test fails to cleanup, the
        # resource group must be manually deleted in order to re-run.
        endpoint_name = 'cdn-cli-test'
        origin = 'www.example.com'
        self.endpoint_create_cmd(resource_group, endpoint_name, profile_name, origin)

        list_checks = [JMESPathCheck('length(@)', 0)]
        self.custom_domain_list_cmd(resource_group, profile_name, endpoint_name, checks=list_checks)

        custom_domain_name = self.create_random_name(prefix='customdomain', length=20)
        hostname = custom_domain_name + '.cdn-cli-test.azfdtest.xyz'
        checks = [JMESPathCheck('name', custom_domain_name),
                  JMESPathCheck('hostName', hostname),
                  JMESPathCheck('customHttpsParameters', None)]
        try:
            self.custom_domain_create_cmd(group=resource_group,
                                          profile_name=profile_name,
                                          endpoint_name=endpoint_name,
                                          name=custom_domain_name,
                                          hostname=hostname,
                                          checks=checks)
        except ErrorResponseException as err:
            if err.status_code != 400:
                raise err
            hostname = custom_domain_name + '.cdn-cli-test-dogfood.azfdtest.xyz'
            self.custom_domain_create_cmd(group=resource_group,
                                          profile_name=profile_name,
                                          endpoint_name=endpoint_name,
                                          name=custom_domain_name,
                                          hostname=hostname,
                                          checks=checks)

        list_checks = [JMESPathCheck('length(@)', 1)]
        self.custom_domain_list_cmd(resource_group, profile_name, endpoint_name, checks=list_checks)

        self.custom_domain_delete_cmd(resource_group, profile_name, endpoint_name, custom_domain_name)

        list_checks = [JMESPathCheck('length(@)', 0)]
        self.custom_domain_list_cmd(resource_group, profile_name, endpoint_name, checks=list_checks)

    @ResourceGroupPreparer()
    def test_cdn_custom_domain_https_akamai(self, resource_group):
        profile_name = 'profile123'
        self.endpoint_list_cmd(resource_group, profile_name, expect_failure=True)

        self.profile_create_cmd(resource_group, profile_name, sku=SkuName.standard_akamai.value)
        # Endpoint name and custom domain hostname are hard-coded because of
        # custom domain CNAME requirement. If test fails to cleanup, the
        # resource group must be manually deleted in order to re-run.
        endpoint_name = 'cdn-cli-test-2'
        origin = 'www.example.com'
        endpoint = self.endpoint_create_cmd(resource_group, endpoint_name, profile_name, origin).get_output_in_json()

        # Create custom domain
        custom_domain_name = self.create_random_name(prefix='customdomain', length=20)
        hostname = custom_domain_name + '.cdn-cli-test-2.azfdtest.xyz'
        # Use alternate hostname for dogfood.
        if '.azureedge-test.net' in endpoint['hostName']:
            hostname = custom_domain_name + '.cdn-cli-test-2-df.azfdtest.xyz'
        self.custom_domain_create_cmd(resource_group, profile_name, endpoint_name, custom_domain_name, hostname)
        checks = [JMESPathCheck('name', custom_domain_name),
                  JMESPathCheck('hostName', hostname),
                  JMESPathCheck('customHttpsParameters', None)]
        self.custom_domain_show_cmd(resource_group, profile_name, endpoint_name, custom_domain_name, checks=checks)

        checks = [JMESPathCheck('name', custom_domain_name),
                  JMESPathCheck('hostName', hostname),
                  JMESPathCheck('customHttpsProvisioningState', 'Enabling'),
                  JMESPathCheck('customHttpsProvisioningSubstate', 'SubmittingDomainControlValidationRequest')]
        self.custom_domain_enable_https_command(resource_group,
                                                profile_name,
                                                endpoint_name,
                                                custom_domain_name,
                                                checks=checks)

        with self.assertRaises(ErrorResponseException):
            self.custom_domain_disable_https_cmd(resource_group, profile_name, endpoint_name, custom_domain_name)

    @ResourceGroupPreparer()
    def test_cdn_custom_domain_https_verizon(self, resource_group):
        profile_name = 'profile123'
        self.endpoint_list_cmd(resource_group, profile_name, expect_failure=True)

        self.profile_create_cmd(resource_group, profile_name, sku=SkuName.standard_verizon.value)
        # Endpoint name and custom domain hostname are hard-coded because of
        # custom domain CNAME requirement. If test fails to cleanup, the
        # resource group must be manually deleted in order to re-run.
        endpoint_name = 'cdn-cli-test-3'
        origin = 'www.contoso.com'
        endpoint = self.endpoint_create_cmd(resource_group, endpoint_name, profile_name, origin).get_output_in_json()

        custom_domain_name = self.create_random_name(prefix='customdomain', length=20)
        hostname = custom_domain_name + '.cdn-cli-test-3.azfdtest.xyz'
        # Use alternate hostnames for dogfood.
        if '.azureedge-test.net' in endpoint['hostName']:
            hostname = custom_domain_name + '.cdn-cli-test-3-df.azfdtest.xyz'
        self.custom_domain_create_cmd(resource_group, profile_name, endpoint_name, custom_domain_name, hostname)

        checks = [JMESPathCheck('name', custom_domain_name),
                  JMESPathCheck('hostName', hostname),
                  JMESPathCheck('customHttpsParameters', None)]
        self.custom_domain_show_cmd(resource_group, profile_name, endpoint_name, custom_domain_name, checks=checks)

        checks = [JMESPathCheck('name', custom_domain_name),
                  JMESPathCheck('hostName', hostname),
                  JMESPathCheck('customHttpsProvisioningState', 'Enabling'),
                  JMESPathCheck('customHttpsProvisioningSubstate', 'SubmittingDomainControlValidationRequest')]
        self.custom_domain_enable_https_command(resource_group,
                                                profile_name,
                                                endpoint_name,
                                                custom_domain_name,
                                                min_tls_version='None',
                                                checks=checks)

        with self.assertRaises(ErrorResponseException):
            self.custom_domain_disable_https_cmd(resource_group, profile_name, endpoint_name, custom_domain_name)

    @ResourceGroupPreparer()
    def test_cdn_custom_domain_https_msft(self, resource_group):
        profile_name = 'profile123'
        self.endpoint_list_cmd(resource_group, profile_name, expect_failure=True)

        self.profile_create_cmd(resource_group, profile_name, sku=SkuName.standard_microsoft.value)
        # Endpoint name and custom domain hostname are hard-coded because of
        # custom domain CNAME requirement. If test fails to cleanup, the
        # resource group must be manually deleted in order to re-run.
        endpoint_name = 'cdn-cli-test-4'
        origin = 'www.example.com'
        endpoint = self.endpoint_create_cmd(resource_group, endpoint_name, profile_name, origin).get_output_in_json()

        # Create custom domains for CDN managed cert and BYOC
        custom_domain_name = self.create_random_name(prefix='customdomain', length=20)
        byoc_custom_domain_name = self.create_random_name(prefix='customdomain', length=20)
        hostname = custom_domain_name + '.cdn-cli-test-4.azfdtest.xyz'
        byoc_hostname = byoc_custom_domain_name + '.cdn-cli-test-4.azfdtest.xyz'
        # Use alternate hostnames for dogfood.
        if '.azureedge-test.net' in endpoint['hostName']:
            hostname = custom_domain_name + '.cdn-cli-test-4-df.azfdtest.xyz'
            byoc_hostname = byoc_custom_domain_name + '.cdn-cli-test-4-df.azfdtest.xyz'
        self.custom_domain_create_cmd(resource_group, profile_name, endpoint_name, custom_domain_name, hostname)
        self.custom_domain_create_cmd(resource_group, profile_name, endpoint_name, byoc_custom_domain_name, byoc_hostname)

        # Verify the created custom domains don't have custom HTTPS enabled
        checks = [JMESPathCheck('name', custom_domain_name),
                  JMESPathCheck('hostName', hostname),
                  JMESPathCheck('customHttpsParameters', None)]
        self.custom_domain_show_cmd(resource_group, profile_name, endpoint_name, custom_domain_name, checks=checks)
        checks = [JMESPathCheck('name', custom_domain_name),
                  JMESPathCheck('hostName', hostname),
                  JMESPathCheck('customHttpsParameters', None)]
        self.custom_domain_show_cmd(resource_group, profile_name, endpoint_name, custom_domain_name, checks=checks)

        # Enable custom HTTPS with a CDN managed certificate.
        checks = [JMESPathCheck('name', custom_domain_name),
                  JMESPathCheck('hostName', hostname),
                  JMESPathCheck('customHttpsProvisioningState', 'Enabling'),
                  JMESPathCheck('customHttpsProvisioningSubstate', 'SubmittingDomainControlValidationRequest')]
        self.custom_domain_enable_https_command(resource_group,
                                                profile_name,
                                                endpoint_name,
                                                custom_domain_name,
                                                min_tls_version='1.0',
                                                checks=checks)

        # Create a TLS cert to use for BYOC.
        vault_name = self.create_random_name(prefix='keyvault', length=20)
        cert_name = self.create_random_name(prefix='cert', length=20)
        self.byoc_create_keyvault_cert(resource_group, vault_name, cert_name)
        # Get and parse the latest version for the certificate.
        versions = self.byoc_get_keyvault_cert_versions(vault_name, cert_name).get_output_in_json()
        version = versions[0]['id'].split('/')[-1]

        # Enable custom HTTPS with a custom certificate
        checks = [JMESPathCheck('name', byoc_custom_domain_name),
                  JMESPathCheck('hostName', byoc_hostname),
                  JMESPathCheck('customHttpsProvisioningState', 'Enabling'),
                  JMESPathCheck('customHttpsProvisioningSubstate', 'ImportingUserProvidedCertificate')]
        self.custom_domain_enable_https_command(resource_group,
                                                profile_name,
                                                endpoint_name,
                                                byoc_custom_domain_name,
                                                user_cert_subscription_id=self.get_subscription_id(),
                                                user_cert_group_name=resource_group,
                                                user_cert_vault_name=vault_name,
                                                user_cert_secret_name=cert_name,
                                                user_cert_secret_version=version,
                                                user_cert_protocol_type='sni',
                                                checks=checks)

        with self.assertRaises(ErrorResponseException):
            self.custom_domain_disable_https_cmd(resource_group, profile_name, endpoint_name, custom_domain_name)

    @ResourceGroupPreparer()
    def test_cdn_custom_domain_byoc_latest(self, resource_group):
        profile_name = 'profile123'
        self.endpoint_list_cmd(resource_group, profile_name, expect_failure=True)

        self.profile_create_cmd(resource_group, profile_name, sku=SkuName.standard_microsoft.value)
        # Endpoint name and custom domain hostname are hard-coded because of
        # custom domain CNAME requirement. If test fails to cleanup, the
        # resource group must be manually deleted in order to re-run.
        endpoint_name = 'cdn-cli-test-5'
        origin = 'www.example.com'
        endpoint = self.endpoint_create_cmd(resource_group, endpoint_name, profile_name, origin).get_output_in_json()

        # Create custom domain for BYOC
        custom_domain_name = self.create_random_name(prefix='customdomain', length=20)
        hostname = custom_domain_name + '.cdn-cli-test-5.azfdtest.xyz'
        # Use alternate hostname for dogfood.
        if '.azureedge-test.net' in endpoint['hostName']:
            hostname = custom_domain_name + '.cdn-cli-test-5-df.azfdtest.xyz'
        self.custom_domain_create_cmd(resource_group, profile_name, endpoint_name, custom_domain_name, hostname)

        # Verify the created custom domain doesn't have custom HTTPS enabled.
        checks = [JMESPathCheck('name', custom_domain_name),
                  JMESPathCheck('hostName', hostname),
                  JMESPathCheck('customHttpsParameters', None)]
        self.custom_domain_show_cmd(resource_group, profile_name, endpoint_name, custom_domain_name, checks=checks)

        # Create a TLS cert to use for BYOC.
        vault_name = self.create_random_name(prefix='keyvault', length=20)
        cert_name = self.create_random_name(prefix='cert', length=20)
        self.byoc_create_keyvault_cert(resource_group, vault_name, cert_name)

        # Enable custom HTTPS with the custom certificate.
        checks = [JMESPathCheck('name', custom_domain_name),
                  JMESPathCheck('hostName', hostname),
                  JMESPathCheck('customHttpsProvisioningState', 'Enabling'),
                  JMESPathCheck('customHttpsProvisioningSubstate', 'ImportingUserProvidedCertificate')]
        self.custom_domain_enable_https_command(resource_group,
                                                profile_name,
                                                endpoint_name,
                                                custom_domain_name,
                                                user_cert_subscription_id=self.get_subscription_id(),
                                                user_cert_group_name=resource_group,
                                                user_cert_vault_name=vault_name,
                                                user_cert_secret_name=cert_name,
                                                user_cert_protocol_type='sni',
                                                checks=checks)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from __future__ import print_function

import sys


def display(txt):
    """ Output to stderr """
    print(txt, file=sys.stderr)


def output(txt):
    """ Output to stdout """
    print(txt, file=sys.stdout)


def get_print_format(records):
    """Find the best format to display the given list of records in table format"""
    if not records:
        raise ValueError('missing parameter records')

    if not isinstance(records, list):
        raise ValueError('records is not a list')

    size = len(records[0])
    max_len = [0] * size

    col_index = list(range(size))
    for rec in records:
        if len(rec) != size:
            raise ValueError('size of elements in the records set are not equal')

        for i in col_index:
            max_len[i] = max(max_len[i], len(str(rec[i])))

    recommend_format = ''
    for each in max_len:
        recommend_format += '{:' + str(each + 2) + '}'

    return recommend_format, max_len


def print_records(records, print_format=None, title=None, foot_notes=None):
    """Print a list of tuples with a print format."""
    print_format = print_format or get_print_format(records)[0]
    if print_format is None:
        raise ValueError('print format is required')

    print()
    print("Summary" + ': {}'.format(title) if title is not None else '')
    print("==========================")
    for rec in records:
        print(print_format.format(*rec))
    print("==========================")
    if foot_notes:
        for each in foot_notes:
            print('* ' + each)


def print_heading(heading, f=None):
    lines = heading.splitlines()
    header_len = max(len(l) for l in lines)
    print('{0}\n{1}\n{0}'.format('=' * header_len, heading), file=f)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import shutil
import configparser
import enum

from knack.log import get_logger
from knack.config import _ConfigFile

ALL = 'all'   # effective level of local context, ALL means all commands can share this parameter value
LOCAL_CONTEXT_FILE = '.local_context_{}'  # each user has a separate file, an example is .local_context_username
LOCAL_CONTEXT_ON_OFF_CONFIG_SECTION = 'local_context'
LOCAL_CONTEXT_NOTICE = '; This file is used to store local context data.\n'\
                       '; DO NOT modify it manually unless you know it well.\n'
logger = get_logger(__name__)


class LocalContextAction(enum.Enum):
    SET = 1   # action for a parameter in local context, SET means its value will be saved to local context
    GET = 2   # action for a parameter in local context, GET means will read value from local context for this parameter


def _get_current_system_username():
    try:
        import getpass
        return getpass.getuser()
    except Exception:  # pylint: disable=broad-except
        pass
    return None


class AzCLILocalContext:  # pylint: disable=too-many-instance-attributes

    def __init__(self, cli_ctx):
        self.cli_ctx = cli_ctx
        self.config = cli_ctx.config
        self.dir_name = os.path.basename(self.config.config_dir)
        self.username = None
        self.is_on = False
        self.current_dir = None

        # only used in get/set/effective_working_directory function, to avoid calling load files to many times.
        self._local_context_file = None

        self.initialize()

    def initialize(self):
        self.username = _get_current_system_username()
        self.is_on = self.config.getboolean(LOCAL_CONTEXT_ON_OFF_CONFIG_SECTION, self.username, False) \
            if self.username else False

        try:
            self.current_dir = os.getcwd()
        except FileNotFoundError:
            if self.is_on:
                logger.warning('The working directory has been deleted or recreated. Parameter persistence is ignored.')

        if self.is_on:
            self._local_context_file = self._get_local_context_file()

    def _get_local_context_file_name(self):
        return LOCAL_CONTEXT_FILE.format(self.username)

    def _load_local_context_files(self, recursive=False):
        local_context_files = []
        if self.username and self.current_dir:
            current_dir = self.current_dir
            while current_dir:
                dir_path = os.path.join(current_dir, self.dir_name)
                file_path = os.path.join(dir_path, self._get_local_context_file_name())
                if os.path.isfile(file_path) and os.access(file_path, os.R_OK) and os.access(file_path, os.W_OK):
                    local_context_files.append(_ConfigFile(dir_path, file_path, LOCAL_CONTEXT_NOTICE))
                    if not recursive:
                        break   # load only one local context
                # Stop if already in root drive
                if current_dir == os.path.dirname(current_dir):
                    break
                current_dir = os.path.dirname(current_dir)
        return local_context_files

    def _get_local_context_file(self):
        local_context_files = self._load_local_context_files(recursive=False)
        if len(local_context_files) == 1:
            return local_context_files[0]
        return None

    def effective_working_directory(self):
        return os.path.dirname(self._local_context_file.config_dir) if self._local_context_file else ''

    def get(self, command, argument):
        if self.is_on and self._local_context_file:
            command_parts = command.split()
            while True:
                section = ' '.join(command_parts) if command_parts else ALL
                try:
                    return self._local_context_file.get(section.lower(), argument)
                except (configparser.NoSectionError, configparser.NoOptionError):
                    pass
                if not command_parts:
                    break
                command_parts = command_parts[:-1]
        return None

    def set(self, scopes, argument, value):
        if self.is_on and self.username and self.current_dir:
            if self._local_context_file is None:
                file_path = os.path.join(self.current_dir, self.dir_name, self._get_local_context_file_name())
                dir_path = os.path.join(self.current_dir, self.dir_name)
                self._local_context_file = _ConfigFile(dir_path, file_path, LOCAL_CONTEXT_NOTICE)

            for scope in scopes:
                self._local_context_file.set_value(scope.lower(), argument, value)

    def turn_on(self):
        self.config.set_value(LOCAL_CONTEXT_ON_OFF_CONFIG_SECTION, self.username, 'on')
        self.is_on = self.config.getboolean(LOCAL_CONTEXT_ON_OFF_CONFIG_SECTION, self.username, False)
        self._local_context_file = self._get_local_context_file()

    def turn_off(self):
        self.config.remove_option(LOCAL_CONTEXT_ON_OFF_CONFIG_SECTION, self.username)
        self.is_on = self.config.getboolean(LOCAL_CONTEXT_ON_OFF_CONFIG_SECTION, self.username, False)
        self._local_context_file = None

    def delete_file(self, recursive=False):
        local_context_files = self._load_local_context_files(recursive=recursive)
        for local_context_file in local_context_files:
            try:
                os.remove(local_context_file.config_path)
                parent_dir = os.path.dirname(local_context_file.config_path)
                if not os.listdir(parent_dir):
                    shutil.rmtree(parent_dir)
                logger.warning('Parameter persistence file in working directory %s is deleted.',
                               os.path.dirname(local_context_file.config_dir))
            except Exception:  # pylint: disable=broad-except
                logger.warning('Fail to delete parameter persistence file in working directory %s',
                               os.path.dirname(local_context_file.config_dir))

    def clear(self, recursive=False):
        local_context_files = self._load_local_context_files(recursive=recursive)
        for local_context_file in local_context_files:
            local_context_file.clear()
            logger.warning('Parameter persistence information in working directory %s is cleared.',
                           os.path.dirname(local_context_file.config_dir))

    def delete(self, names=None):
        local_context_file = self._get_local_context_file()
        if local_context_file:
            for scope in local_context_file.sections():
                for name in names:
                    local_context_file.remove_option(scope, name)
        logger.warning('Parameter persistence value is deleted. You can run `az config param-persist show` to show all '
                       'available values.')

    def get_value(self, names=None):
        result = {}

        local_context_file = self._get_local_context_file()
        if not local_context_file:
            return result

        for scope in local_context_file.sections():
            try:
                if names is None:
                    for name, value in local_context_file.items(scope):  # may raise NoSectionError
                        if scope not in result:
                            result[scope] = {}
                        result[scope][name] = value
                else:
                    for name in names:
                        value = local_context_file.get(scope, name)  # may raise NoOptionError
                        if scope not in result:
                            result[scope] = {}
                        result[scope][name] = value
            except (configparser.NoSectionError, configparser.NoOptionError):
                pass
        return result


class LocalContextAttribute:
    # pylint: disable=too-few-public-methods
    def __init__(self, name, actions, scopes=None):
        """ Local Context Attribute arguments

        :param name: Argument name in local context. Make sure it is consistent for SET and GET.
        :type name: str
        :param actions: Which action should be taken for local context. Allowed values: SET, GET
        :type actions: list
        :param scopes: The effective commands or command groups of this argument when saved to local context.
        :type scopes: list
        """
        self.name = name

        if isinstance(actions, str):
            actions = [actions]
        self.actions = actions

        if isinstance(scopes, str):
            scopes = [scopes]
        if scopes is None and LocalContextAction.SET in actions:
            scopes = [ALL]
        self.scopes = scopes
